@@ -1,6 +1,6 @@
-/* gdaui-numeric-entry.c
- *
+/*
  * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -90,6 +90,7 @@ gdaui_numeric_entry_get_type (void)
 			sizeof (GdauiNumericEntry),
 			0,		/* n_preallocs */
 			(GInstanceInitFunc) gdaui_numeric_entry_init,
+			0
 		};
 		
 		type = g_type_register_static (GDAUI_TYPE_ENTRY, "GdauiNumericEntry", &type_info, 0);
