@@ -1,21 +1,21 @@
-/* gdaui-data-proxy-info.c
+/*
+ * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * Copyright (C) 2006 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -47,6 +47,8 @@ static void data_proxy_proxy_changed_cb (GdauiDataProxy *data_proxy, GdaDataProx
 static void proxy_changed_cb (GdaDataProxy *proxy, GdauiDataProxyInfo *info);
 static void proxy_sample_changed_cb (GdaDataProxy *proxy, gint sample_start, gint sample_end, GdauiDataProxyInfo *info);
 static void proxy_row_changed_cb (GdaDataProxy *proxy, gint row, GdauiDataProxyInfo *info);
+static void proxy_reset_cb (GdaDataProxy *wid, GdauiDataProxyInfo *info);
+
 static void raw_grid_selection_changed_cb (GdauiRawGrid *grid, GdauiDataProxyInfo *info);
 
 
@@ -58,8 +60,14 @@ struct _GdauiDataProxyInfoPriv
 	GdauiDataProxyInfoFlag flags; /* ORed values. */
 
 	GtkUIManager      *uimanager;
+	GtkActionGroup    *agroup; /* no ref held! */
+	guint              merge_id_row_modif;
+	guint              merge_id_row_move;
+	guint              merge_id_chunck_change;
 
 	GtkWidget         *buttons_bar;
+	gboolean           buttons_bar_from_ui;
+	GtkWidget         *tool_item;
 	GtkWidget         *current_sample;
 	GtkWidget         *row_spin;
 
@@ -73,7 +81,8 @@ static GObjectClass *parent_class = NULL;
 enum {
 	PROP_0,
 	PROP_DATA_PROXY,
-	PROP_FLAGS
+	PROP_FLAGS,
+	PROP_UI_MANAGER
 };
 
 GType
@@ -91,7 +100,8 @@ gdaui_data_proxy_info_get_type (void)
 			NULL,
 			sizeof (GdauiDataProxyInfo),
 			0,
-			(GInstanceInitFunc) gdaui_data_proxy_info_init
+			(GInstanceInitFunc) gdaui_data_proxy_info_init,
+			0
 		};
 
 		type = g_type_register_static (GTK_TYPE_HBOX, "GdauiDataProxyInfo", &info, 0);
@@ -120,6 +130,17 @@ gdaui_data_proxy_info_class_init (GdauiDataProxyInfoClass *klass)
                                          g_param_spec_flags ("flags", NULL, NULL, GDAUI_TYPE_DATA_PROXY_INFO_FLAG,
 							     GDAUI_DATA_PROXY_INFO_CURRENT_ROW,
 							     G_PARAM_READABLE | G_PARAM_WRITABLE));
+	/**
+	 * GdauiDataProxyInfo:ui-manager:
+	 *
+	 * Use this property to obtain the #GtkUIManager object internally used (to add new actions
+	 * for example).
+	 *
+	 * Since: 4.2.9
+	 */
+	g_object_class_install_property (object_class, PROP_UI_MANAGER,
+                                         g_param_spec_object ("ui-manager", NULL, NULL, GTK_TYPE_UI_MANAGER,
+							      G_PARAM_READABLE));
 }
 
 static void
@@ -129,6 +150,8 @@ gdaui_data_proxy_info_init (GdauiDataProxyInfo *wid)
 	wid->priv->data_proxy = NULL;
 	wid->priv->proxy = NULL;
 	wid->priv->row_spin = NULL;
+	wid->priv->buttons_bar = NULL;
+	wid->priv->tool_item = NULL;
 }
 
 /**
@@ -171,7 +194,6 @@ data_proxy_destroyed_cb (GdauiDataProxy *wid, GdauiDataProxyInfo *info)
 	info->priv->data_proxy = NULL;
 }
 
-
 static void
 release_proxy (GdauiDataProxyInfo *info)
 {
@@ -181,6 +203,8 @@ release_proxy (GdauiDataProxyInfo *info)
 					      G_CALLBACK (proxy_sample_changed_cb), info);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (info->priv->proxy),
 					      G_CALLBACK (proxy_row_changed_cb), info);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (info->priv->proxy),
+					      G_CALLBACK (proxy_reset_cb), info);
 	g_object_unref (info->priv->proxy);
 	info->priv->proxy = NULL;
 }
@@ -196,7 +220,7 @@ release_iter (GdauiDataProxyInfo *info)
 }
 
 static void
-data_proxy_proxy_changed_cb (GdauiDataProxy *data_proxy, GdaDataProxy *proxy, GdauiDataProxyInfo *info)
+data_proxy_proxy_changed_cb (GdauiDataProxy *data_proxy, G_GNUC_UNUSED GdaDataProxy *proxy, GdauiDataProxyInfo *info)
 {
 	g_object_set (G_OBJECT (info), "data-proxy", data_proxy, NULL);
 }
@@ -220,6 +244,19 @@ gdaui_data_proxy_info_dispose (GObject *object)
 		if (info->priv->idle_id)
 			g_source_remove (info->priv->idle_id);
 
+		if (info->priv->uimanager) {
+			if (info->priv->merge_id_row_modif)
+				gtk_ui_manager_remove_ui (info->priv->uimanager,
+							  info->priv->merge_id_row_modif);
+			if (info->priv->merge_id_row_move)
+				gtk_ui_manager_remove_ui (info->priv->uimanager,
+							  info->priv->merge_id_row_move);
+			if (info->priv->merge_id_chunck_change)
+				gtk_ui_manager_remove_ui (info->priv->uimanager,
+							  info->priv->merge_id_chunck_change);
+			g_object_unref (info->priv->uimanager);
+		}
+
 		/* the private area itself */
 		g_free (info->priv);
 		info->priv = NULL;
@@ -275,6 +312,9 @@ gdaui_data_proxy_info_set_property (GObject *object,
 							  G_CALLBACK (proxy_row_changed_cb), info);
 					g_signal_connect (G_OBJECT (proxy), "row-removed",
 							  G_CALLBACK (proxy_row_changed_cb), info);
+					g_signal_connect (G_OBJECT (proxy), "reset",
+							  G_CALLBACK (proxy_reset_cb), info);
+
 
 					/* iter */
 					iter = gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR
@@ -291,17 +331,6 @@ gdaui_data_proxy_info_set_property (GObject *object,
                         break;
                 case PROP_FLAGS:
 			info->priv->flags = g_value_get_flags (value);
-			if (info->priv->row_spin) {
-				gtk_widget_destroy (info->priv->row_spin);
-				info->priv->row_spin = NULL;
-			}
-			if (info->priv->buttons_bar) {
-				gtk_widget_destroy (info->priv->buttons_bar);
-				info->priv->buttons_bar = NULL;
-			}
-			if (info->priv->current_sample)
-				info->priv->current_sample = NULL;
-
 			modif_buttons_make (info);
 			modif_buttons_update (info);
                         break;
@@ -324,11 +353,14 @@ gdaui_data_proxy_info_get_property (GObject *object,
         if (info->priv) {
                 switch (param_id) {
 		case PROP_DATA_PROXY:
-			g_value_set_pointer (value, info->priv->data_proxy);
+			g_value_set_object (value, info->priv->data_proxy);
 			break;
 		case PROP_FLAGS:
 			g_value_set_flags (value, info->priv->flags);
 			break;
+		case PROP_UI_MANAGER:
+			g_value_set_object (value, info->priv->uimanager);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
@@ -338,31 +370,39 @@ gdaui_data_proxy_info_get_property (GObject *object,
 
 
 static void
-proxy_changed_cb (GdaDataProxy *proxy, GdauiDataProxyInfo *info)
+proxy_changed_cb (G_GNUC_UNUSED GdaDataProxy *proxy, G_GNUC_UNUSED GdauiDataProxyInfo *info)
 {
 	modif_buttons_update (info);
 }
 
 static void
-proxy_sample_changed_cb (GdaDataProxy *proxy, gint sample_start, gint sample_end, GdauiDataProxyInfo *info)
+proxy_sample_changed_cb (G_GNUC_UNUSED GdaDataProxy *proxy, G_GNUC_UNUSED gint sample_start,
+			 G_GNUC_UNUSED gint sample_end, G_GNUC_UNUSED GdauiDataProxyInfo *info)
 {
 	modif_buttons_update (info);
 }
 
 static void
-proxy_row_changed_cb (GdaDataProxy *proxy, gint row, GdauiDataProxyInfo *info)
+proxy_row_changed_cb (G_GNUC_UNUSED GdaDataProxy *proxy, G_GNUC_UNUSED gint row, GdauiDataProxyInfo *info)
 {
 	modif_buttons_update (info);
 }
 
 static void
-iter_row_changed_cb (GdaDataModelIter *iter, gint row, GdauiDataProxyInfo *info)
+proxy_reset_cb (G_GNUC_UNUSED GdaDataProxy *wid, GdauiDataProxyInfo *info)
+{
+	modif_buttons_make (info);
+	modif_buttons_update (info);
+}
+
+static void
+iter_row_changed_cb (G_GNUC_UNUSED GdaDataModelIter *iter, G_GNUC_UNUSED gint row, GdauiDataProxyInfo *info)
 {
 	modif_buttons_update (info);
 }
 
 static void
-raw_grid_selection_changed_cb (GdauiRawGrid *grid, GdauiDataProxyInfo *info)
+raw_grid_selection_changed_cb (G_GNUC_UNUSED GdauiRawGrid *grid, GdauiDataProxyInfo *info)
 {
 	modif_buttons_update (info);
 }
@@ -372,37 +412,52 @@ raw_grid_selection_changed_cb (GdauiRawGrid *grid, GdauiDataProxyInfo *info)
  * Modification buttons (Commit changes, Reset info, New entry, Delete)
  *
  */
+static const gchar *ui_base =
+	"<ui>"
+	"  <toolbar name='ToolBar'>"
+	"    <placeholder name='RowModif'/>"
+	"    <placeholder name='RowModifExtension'/>"
+	"    <placeholder name='RowMove'/>"
+	"    <placeholder name='ChunkChange'/>"
+	"    <toolitem action='ActionFilter'/>"
+	"  </toolbar>"
+	"</ui>";
+
 static const gchar *ui_row_modif =
 	"<ui>"
-	"  <toolbar  name='ToolBar'>"
-	"    <toolitem action='ActionNew'/>"
-	"    <toolitem action='ActionDelete'/>"
-	"    <toolitem action='ActionUndelete'/>"
-	"    <toolitem action='ActionCommit'/>"
-	"    <toolitem action='ActionReset'/>"
+	"  <toolbar name='ToolBar'>"
+	"    <placeholder name='RowModif'>"
+	"      <toolitem action='ActionNew'/>"
+	"      <toolitem action='ActionDelete'/>"
+	"      <toolitem action='ActionCommit'/>"
+	"      <toolitem action='ActionReset'/>"
+	"    </placeholder>"
 	"  </toolbar>"
 	"</ui>";
 static const gchar *ui_row_move =
 	"<ui>"
-	"  <toolbar  name='ToolBar'>"
-	"    <toolitem action='ActionFirstRecord'/>"
-	"    <toolitem action='ActionPrevRecord'/>"
-	"    <toolitem action='ActionNextRecord'/>"
-	"    <toolitem action='ActionLastRecord'/>"
-	"    <toolitem action='ActionFilter'/>"
+	"  <toolbar name='ToolBar'>"
+	"    <placeholder name='RowMove'>"
+	"      <toolitem action='ActionFirstRecord'/>"
+	"      <toolitem action='ActionPrevRecord'/>"
+	"      <toolitem action='ActionNextRecord'/>"
+	"      <toolitem action='ActionLastRecord'/>"
+	"    </placeholder>"
 	"  </toolbar>"
 	"</ui>";
 static const gchar *ui_chunck_change =
 	"<ui>"
-	"  <toolbar  name='ToolBar'>"
-	"    <toolitem action='ActionFirstChunck'/>"
-	"    <toolitem action='ActionPrevChunck'/>"
-	"    <toolitem action='ActionNextChunck'/>"
-	"    <toolitem action='ActionLastChunck'/>"
-	"    <toolitem action='ActionFilter'/>"
+	"  <toolbar name='ToolBar'>"
+	"    <placeholder name='ChunkChange'>"
+	"      <toolitem action='ActionFirstChunck'/>"
+	"      <toolitem action='ActionPrevChunck'/>"
+	"      <toolitem action='ActionNextChunck'/>"
+	"      <toolitem action='ActionLastChunck'/>"
+	"    </placeholder>"
 	"  </toolbar>"
 	"</ui>";
 
+
 static void row_spin_changed_cb (GtkSpinButton *spin, GdauiDataProxyInfo *info);
 static void
 modif_buttons_make (GdauiDataProxyInfo *info)
@@ -416,6 +471,7 @@ modif_buttons_make (GdauiDataProxyInfo *info)
                 gtk_rc_parse_string ("style \"gdaui-data-proxy-info-style\"\n"
                                      "{\n"
                                      "GtkToolbar::shadow-type = GTK_SHADOW_NONE\n"
+				     "GtkSpinButton::shadow-type = GTK_SHADOW_NONE\n"
                                      "xthickness = 0\n"
                                      "ythickness = 0\n"
                                      "}\n"
@@ -425,42 +481,117 @@ modif_buttons_make (GdauiDataProxyInfo *info)
 	if (! info->priv->data_proxy)
 		return;
 
+	if (info->priv->tool_item) {
+		/* remove tool_item from toolbar */
+		gtk_container_remove (GTK_CONTAINER (info->priv->buttons_bar),
+				      info->priv->tool_item);
+		gtk_widget_unparent (info->priv->tool_item);
+	}
+
+	gboolean ensure_update = FALSE;
+
+	if (info->priv->uimanager) {
+		if (info->priv->merge_id_row_modif) {
+			gtk_ui_manager_remove_ui (info->priv->uimanager, info->priv->merge_id_row_modif);
+			info->priv->merge_id_row_modif = 0;
+		}
+		if (info->priv->merge_id_row_move) {
+			gtk_ui_manager_remove_ui (info->priv->uimanager, info->priv->merge_id_row_move);
+			info->priv->merge_id_row_move = 0;
+		}
+		if (info->priv->merge_id_chunck_change) {
+			gtk_ui_manager_remove_ui (info->priv->uimanager, info->priv->merge_id_chunck_change);
+			info->priv->merge_id_chunck_change = 0;
+		}
+		gtk_ui_manager_remove_action_group (info->priv->uimanager, info->priv->agroup);
+		info->priv->agroup = NULL;
+	}
+	else {
+		info->priv->uimanager = gtk_ui_manager_new ();	
+		gtk_ui_manager_add_ui_from_string (info->priv->uimanager, ui_base, -1, NULL);
+	}
+
+	info->priv->agroup = gdaui_data_proxy_get_actions_group (info->priv->data_proxy);
+	gtk_ui_manager_insert_action_group (info->priv->uimanager, info->priv->agroup, 0);
+
 	if (flags & (GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS |
 		     GDAUI_DATA_PROXY_INFO_ROW_MOVE_BUTTONS |
 		     GDAUI_DATA_PROXY_INFO_CHUNCK_CHANGE_BUTTONS)) {
-		GtkActionGroup *actions;
 		GtkUIManager *ui;
-
-		actions = gdaui_data_proxy_get_actions_group (info->priv->data_proxy);
-		ui = gtk_ui_manager_new ();
-		gtk_ui_manager_insert_action_group (ui, actions, 0);
+		ui = info->priv->uimanager;
 		if (flags & GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS)
-			gtk_ui_manager_add_ui_from_string (ui, ui_row_modif, -1, NULL);
+			info->priv->merge_id_row_modif = gtk_ui_manager_add_ui_from_string (ui, ui_row_modif,
+											    -1, NULL);
 		if (flags & GDAUI_DATA_PROXY_INFO_ROW_MOVE_BUTTONS)
-			gtk_ui_manager_add_ui_from_string (ui, ui_row_move, -1, NULL);
+			info->priv->merge_id_row_move = gtk_ui_manager_add_ui_from_string (ui, ui_row_move,
+											   -1, NULL);
 		if (flags & GDAUI_DATA_PROXY_INFO_CHUNCK_CHANGE_BUTTONS)
-			gtk_ui_manager_add_ui_from_string (ui, ui_chunck_change, -1, NULL);
-
-		info->priv->uimanager = ui;
-		info->priv->buttons_bar = gtk_ui_manager_get_widget (ui, "/ToolBar");
-		gtk_toolbar_set_icon_size (GTK_TOOLBAR (info->priv->buttons_bar), GTK_ICON_SIZE_SMALL_TOOLBAR);
-		g_object_set (G_OBJECT (info->priv->buttons_bar), "toolbar-style", GTK_TOOLBAR_ICONS, NULL);
-		gtk_widget_set_name (info->priv->buttons_bar, "gdaui-data-proxy-info");
-		gtk_box_pack_start (GTK_BOX (info), info->priv->buttons_bar, TRUE, TRUE, 0);
+			info->priv->merge_id_chunck_change = gtk_ui_manager_add_ui_from_string (ui, ui_chunck_change,
+												-1, NULL);
 	}
+
+	/* get rid of previous toolbar if any */
+	if (info->priv->buttons_bar) {
+		if (gtk_widget_get_parent (info->priv->buttons_bar)) {
+			gtk_container_remove (GTK_CONTAINER (info), info->priv->buttons_bar);
+			gtk_widget_unparent (info->priv->buttons_bar);
+		}
+		if (info->priv->buttons_bar_from_ui)
+			g_object_unref (info->priv->buttons_bar);
+		else
+			gtk_widget_destroy (info->priv->buttons_bar);
+		info->priv->buttons_bar = NULL;
+	}
+
+	/* create new toolbar */
+	GtkUIManager *ui;
+	ui = info->priv->uimanager;
+	info->priv->buttons_bar = gtk_ui_manager_get_widget (ui, "/ToolBar");
+	if (info->priv->buttons_bar)
+		info->priv->buttons_bar_from_ui = TRUE;
 	else {
 		info->priv->buttons_bar = gtk_toolbar_new ();
-		gtk_box_pack_start (GTK_BOX (info), info->priv->buttons_bar, TRUE, TRUE, 0);
+		info->priv->buttons_bar_from_ui = FALSE;
 	}
+	g_object_ref_sink (info->priv->buttons_bar);
+	gtk_toolbar_set_icon_size (GTK_TOOLBAR (info->priv->buttons_bar), GTK_ICON_SIZE_MENU);
+	g_object_set (G_OBJECT (info->priv->buttons_bar), "toolbar-style", GTK_TOOLBAR_ICONS, NULL);
+	gtk_widget_set_name (info->priv->buttons_bar, "gdaui-data-proxy-info");
+	gtk_box_pack_start (GTK_BOX (info), info->priv->buttons_bar, TRUE, TRUE, 0);
 	gtk_widget_show (info->priv->buttons_bar);
 
 	if (flags & GDAUI_DATA_PROXY_INFO_CURRENT_ROW) {
+		if (info->priv->tool_item) {
+			/* remove the current contents */
+			gtk_widget_destroy (gtk_bin_get_child (GTK_BIN (info->priv->tool_item)));
+			info->priv->row_spin = NULL;
+			info->priv->current_sample = NULL;
+			gtk_toolbar_insert (GTK_TOOLBAR (info->priv->buttons_bar),
+					    GTK_TOOL_ITEM (info->priv->tool_item), -1);
+		}
+		else {
+			GtkToolItem *ti;
+			ti = gtk_tool_item_new  ();
+			gtk_toolbar_insert (GTK_TOOLBAR (info->priv->buttons_bar), ti, -1);
+			info->priv->tool_item = GTK_WIDGET (g_object_ref (G_OBJECT (ti)));
+		}
+
 		GtkWidget *toolwid;
+		PangoContext *pc;
+		PangoFontDescription *fd, *fdc;
+		pc = gtk_widget_get_pango_context (GTK_WIDGET (info));
+		fd = pango_context_get_font_description (pc);
+		fdc = pango_font_description_copy (fd);
+		pango_font_description_set_size (fdc,
+						 pango_font_description_get_size (fd) * .8);
+
 		if (flags & GDAUI_DATA_PROXY_INFO_ROW_MOVE_BUTTONS) {
 			toolwid = gtk_hbox_new (FALSE, 0);
 
 			/* read-write spin counter (mainly for forms) */
 			wid = gtk_spin_button_new_with_range (0, 1, 1);
+			gtk_widget_modify_font (wid, fdc);
+			gtk_widget_set_name (wid, "gdaui-data-proxy-info");
 			gtk_spin_button_set_digits (GTK_SPIN_BUTTON (wid), 0);
 			gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (wid), TRUE);
 			gtk_box_pack_start (GTK_BOX (toolwid), wid, FALSE, TRUE, 2);
@@ -471,22 +602,25 @@ modif_buttons_make (GdauiDataProxyInfo *info)
 
 			/* rows counter */
 			wid = gtk_label_new (" /?");
+			gtk_widget_modify_font (wid, fdc);
 			info->priv->current_sample = wid;
 			gtk_box_pack_start (GTK_BOX (toolwid), wid, FALSE, FALSE, 2);
 		}
 		else {
 			/* read-only counter (mainly for grids) */
 			wid = gtk_label_new ("? - ? /?");
+			gtk_widget_modify_font (wid, fdc);
 			info->priv->current_sample = wid;
 			toolwid = wid;
 		}
 
-		GtkToolItem *ti;
-		ti = gtk_tool_item_new  ();
-		gtk_container_add (GTK_CONTAINER (ti), toolwid);
-		gtk_toolbar_insert (GTK_TOOLBAR (info->priv->buttons_bar), ti, -1);
-		gtk_widget_show_all (GTK_WIDGET (ti));
+		pango_font_description_free (fdc);
+
+		gtk_container_add (GTK_CONTAINER (info->priv->tool_item), toolwid);
+		gtk_widget_show_all (info->priv->tool_item);
 	}
+	else if (info->priv->tool_item)
+		gtk_widget_hide (info->priv->tool_item);
 }
 
 static void
@@ -585,7 +719,9 @@ idle_modif_buttons_update (GdauiDataProxyInfo *info)
 						str = g_strdup_printf (" / %d", proxy_rows);
 				}
 				BLOCK_SPIN;
-				gtk_spin_button_set_range (GTK_SPIN_BUTTON (info->priv->row_spin), 1, proxy_rows);
+				gtk_spin_button_set_range (GTK_SPIN_BUTTON (info->priv->row_spin),
+							   proxy_rows > 0 ? 1 : 0,
+							   proxy_rows);
 				if (row >= 0)
 					if (gtk_spin_button_get_value (GTK_SPIN_BUTTON (info->priv->row_spin)) != row+1)
 						gtk_spin_button_set_value (GTK_SPIN_BUTTON (info->priv->row_spin), row+1);
@@ -667,34 +803,42 @@ idle_modif_buttons_update (GdauiDataProxyInfo *info)
 			GdauiDataProxyWriteMode mode;
 			mode = gdaui_data_proxy_get_write_mode (info->priv->data_proxy);
 
-			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionCommit");
+			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/RowModif/ActionCommit");
 			g_object_set (G_OBJECT (action), "sensitive", changed ? TRUE : FALSE, NULL);
 			if (mode == GDAUI_DATA_PROXY_WRITE_ON_VALUE_CHANGE)
 				gtk_action_set_visible (action, FALSE);
 			else
 				gtk_action_set_visible (action, TRUE);
 
-			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionReset");
+			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/RowModif/ActionReset");
 			g_object_set (G_OBJECT (action), "sensitive", changed ? TRUE : FALSE, NULL);
 			if (mode == GDAUI_DATA_PROXY_WRITE_ON_VALUE_CHANGE)
 				gtk_action_set_visible (action, FALSE);
 			else
 				gtk_action_set_visible (action, TRUE);
 
-			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionNew");
+			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/RowModif/ActionNew");
 			g_object_set (G_OBJECT (action), "sensitive",
 				      flags & GDA_DATA_MODEL_ACCESS_INSERT ? TRUE : FALSE, NULL);
 
-			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionDelete");
-			wrows = is_inserted ||
-				((flags & GDA_DATA_MODEL_ACCESS_DELETE) &&
-				 (force_del_btn || (! to_be_deleted && has_selection)));
-			g_object_set (G_OBJECT (action), "sensitive", wrows, NULL);
+			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/RowModif/ActionDelete");
+			gtk_action_block_activate (action);
+			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), to_be_deleted);
+			gtk_action_unblock_activate (action);
 
-			action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionUndelete");
-			wrows = (flags & GDA_DATA_MODEL_ACCESS_DELETE) &&
-				(force_undel_btn || (to_be_deleted && has_selection));
-			g_object_set (G_OBJECT (action), "sensitive", wrows, NULL);
+			if (to_be_deleted) {
+				wrows = (flags & GDA_DATA_MODEL_ACCESS_DELETE) &&
+					(force_undel_btn || has_selection);
+				g_object_set (G_OBJECT (action), "sensitive", wrows, NULL);
+				gtk_action_set_tooltip (action, _("Undelete the selected entry"));
+			}
+			else {
+				wrows = is_inserted ||
+					((flags & GDA_DATA_MODEL_ACCESS_DELETE) &&
+					 (force_del_btn || has_selection));
+				g_object_set (G_OBJECT (action), "sensitive", wrows, NULL);
+				gtk_action_set_tooltip (action, _("Delete the selected entry"));
+			}
 
 			if ((mode == GDAUI_DATA_PROXY_WRITE_ON_ROW_CHANGE) ||
 			    (mode == GDAUI_DATA_PROXY_WRITE_ON_VALUE_CHANGE) ||
@@ -707,14 +851,14 @@ idle_modif_buttons_update (GdauiDataProxyInfo *info)
 
 	/* current row moving */
 	if (info->priv->flags & GDAUI_DATA_PROXY_INFO_ROW_MOVE_BUTTONS) {
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionFirstRecord");
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/RowMove/ActionFirstRecord");
 		g_object_set (G_OBJECT (action), "sensitive", (row <= 0) ? FALSE : TRUE, NULL);
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionPrevRecord");
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/RowMove/ActionPrevRecord");
 		g_object_set (G_OBJECT (action), "sensitive", (row <= 0) ? FALSE : TRUE, NULL);
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionNextRecord");
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/RowMove/ActionNextRecord");
 		g_object_set (G_OBJECT (action), "sensitive", (row == proxy_rows -1) || (row < 0) ? FALSE : TRUE,
 			      NULL);
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionLastRecord");
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/RowMove/ActionLastRecord");
 		g_object_set (G_OBJECT (action), "sensitive", (row == proxy_rows -1) || (row < 0) ? FALSE : TRUE,
 			      NULL);
 	}
@@ -723,22 +867,23 @@ idle_modif_buttons_update (GdauiDataProxyInfo *info)
 	if (info->priv->flags & GDAUI_DATA_PROXY_INFO_CHUNCK_CHANGE_BUTTONS) {
 		gboolean abool;
 		wrows = (sample_size > 0) ? TRUE : FALSE;
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionFirstChunck");
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ChunkChange/ActionFirstChunck");
 		g_object_set (G_OBJECT (action), "sensitive", wrows && sample_first_row > 0 ? TRUE : FALSE, NULL);
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionPrevChunck");
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ChunkChange/ActionPrevChunck");
 		g_object_set (G_OBJECT (action), "sensitive", wrows && sample_first_row > 0 ? TRUE : FALSE, NULL);
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionNextChunck");
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ChunkChange/ActionNextChunck");
 		abool = (proxied_rows != -1) ? wrows && (sample_last_row < proxied_rows - 1) : TRUE;
 		g_object_set (G_OBJECT (action), "sensitive", abool, NULL);
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionLastChunck");
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ChunkChange/ActionLastChunck");
 		g_object_set (G_OBJECT (action), "sensitive", wrows && (sample_last_row < proxied_rows - 1), NULL);
 	}
 
 	/* filter */
-	if (info->priv->flags & GDAUI_DATA_PROXY_INFO_NO_FILTER) {
-		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionFilter");
+	action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionFilter");
+	if (info->priv->flags & GDAUI_DATA_PROXY_INFO_NO_FILTER)
 		g_object_set (G_OBJECT (action), "visible", FALSE, NULL);
-	}
+	else
+		g_object_set (G_OBJECT (action), "visible", TRUE, NULL);
 
 	if (info->priv->uimanager)
 		gtk_ui_manager_ensure_update (info->priv->uimanager);
