@@ -1,21 +1,21 @@
-/* utility.c
- *
- * Copyright (C) 2003 - 2009 Vivien Malerba <malerba@gnome-db.org>
+/*
+ * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -28,7 +28,7 @@
 #include <libgda-ui/gdaui-decl.h>
 
 
-/**
+/*
  * _gdaui_utility_entry_build_actions_menu
  * @obj_data:
  * @attrs:
@@ -119,7 +119,7 @@ _gdaui_utility_entry_build_actions_menu (GObject *obj_data, guint attrs, GCallba
 }
 
 
-/**
+/*
  * _gdaui_utility_entry_build_info_colors_array
  * 
  * Creates an array of colors for the different states of an entry:
@@ -132,7 +132,8 @@ _gdaui_utility_entry_build_actions_menu (GObject *obj_data, guint attrs, GCallba
  *
  * Returns: a new array of 6 colors
  */
-GdkColor **_gdaui_utility_entry_build_info_colors_array ()
+GdkColor **
+_gdaui_utility_entry_build_info_colors_array (void)
 {
 	GdkColor **colors;
 	GdkColor *color;
@@ -195,7 +196,7 @@ GdkColor **_gdaui_utility_entry_build_info_colors_array ()
 	return colors;
 }
 
-/**
+/*
  * _gdaui_utility_markup_title
  */
 gchar *
@@ -207,7 +208,7 @@ _gdaui_utility_markup_title (const gchar *title, gboolean optional)
 		return g_strdup_printf ("%s:", title);
 }
 
-/**
+/*
  * _gdaui_utility_proxy_compute_attributes_for_group
  *
  * Computes an attributes from the individual attributes of the values stored in @store and
@@ -239,7 +240,7 @@ _gdaui_utility_proxy_compute_attributes_for_group (GdauiSetGroup *group, GdauiDa
         for (list = group->group->nodes; list; list = list->next) {
 		col = g_slist_index (((GdaSet*)model_iter)->holders, GDA_SET_NODE (list->data)->holder);
                 gtk_tree_model_get (GTK_TREE_MODEL (store), tree_iter,
-                                    DATA_STORE_COL_TO_DELETE, &local_to_del,
+                                    GDAUI_DATA_STORE_COL_TO_DELETE, &local_to_del,
                                     offset + col, &localattr, -1);
 		if (list == group->group->nodes)
 			attributes = localattr;
@@ -254,7 +255,7 @@ _gdaui_utility_proxy_compute_attributes_for_group (GdauiSetGroup *group, GdauiDa
         return attributes;
 }
 
-/**
+/*
  * _gdaui_utility_proxy_compute_values_for_group:
  *
  * Computes a list of values containing the individual values stored in @store and
@@ -274,12 +275,11 @@ _gdaui_utility_proxy_compute_values_for_group (GdauiSetGroup *group, GdauiDataSt
 					       GdaDataModelIter *model_iter, 
 					       GtkTreeIter *tree_iter, gboolean model_values)
 {
-	gint offset;
 	GList *retval = NULL;
+#ifdef PROXY_STORE_EXTRA_VALUES
 	GdaDataProxy *proxy;
-
 	proxy = gdaui_data_store_get_proxy (store);
-	offset = gda_data_proxy_get_proxied_model_n_cols (proxy);
+#endif
 	if (!model_values) {
 		GSList *list;
 		GValue *value;
@@ -293,13 +293,17 @@ _gdaui_utility_proxy_compute_values_for_group (GdauiSetGroup *group, GdauiDataSt
 		}
 	}
 	else {
-		gint col, i, proxy_row;
+		gint col, i;
+#ifdef PROXY_STORE_EXTRA_VALUES
+		gint proxy_row;
+#endif
 		GdauiSetSource *source;
 		const GValue *value;
 		gboolean slow_way = FALSE;
 		gboolean ret_null = FALSE;
-
+#ifdef PROXY_STORE_EXTRA_VALUES
 		proxy_row = gdaui_data_store_get_row_from_iter (store, tree_iter);
+#endif
 		source = group->source;
 		for (i = 0 ; (i < source->shown_n_cols)  && !ret_null; i++) {
 			col = source->shown_cols_index[i];
@@ -393,11 +397,11 @@ create_data_error_dialog (GdauiDataProxy *form, gboolean with_question, gboolean
 	if (can_discard) {
 		msg1 = _("Current modified data is invalid");
 		if (with_question)
-			msg2 =_("You may now choose to correct it, or to discard\n"
+			msg2 =_("You may now choose to correct it, or to discard "
 				"the modifications.\n\n"
 				"What do you wan to do?");
 		else
-			msg2 = _("please correct it and try again, or discard\n"
+			msg2 = _("please correct it and try again, or discard "
 				 "the modifications.");
 	}
 	else {
@@ -416,45 +420,18 @@ create_data_error_dialog (GdauiDataProxy *form, gboolean with_question, gboolean
 						  "<b>%s:</b>\n\n%s", msg1, msg2);
 
 	if (filled_error && filled_error->message) {
-		GtkWidget *exp, *sw, *view;
-		GtkTextBuffer *buffer;
-		GtkTextIter start, end;
-		GtkTextTag *tag;
-		
-		exp = gtk_expander_new (_("Error details"));
+		GtkWidget *label;
 
-		sw = gtk_scrolled_window_new (NULL, NULL);
-		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-						GTK_POLICY_AUTOMATIC,
-						GTK_POLICY_AUTOMATIC);
-
-		view = gtk_text_view_new ();
-		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-		gtk_text_buffer_set_text (buffer, filled_error->message, -1);
-						
-		/* Use a tag to change the color for just one part of the widget */
-		tag = gtk_text_buffer_create_tag (buffer, "monospace",
-						  "family", "monospace", NULL);
-		gtk_text_buffer_get_bounds (buffer, &start, &end);
-		gtk_text_buffer_apply_tag (buffer, tag, &start, &end);
-		
-		gtk_container_add (GTK_CONTAINER (sw), view);
-		gtk_container_add (GTK_CONTAINER (exp), sw);
-#if GTK_CHECK_VERSION(2,18,0)
+		label = gtk_label_new (filled_error->message);
 		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
-				    exp, TRUE, TRUE, 0);
-#else
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), exp, TRUE, TRUE, 0);
-#endif
-		gtk_widget_show_all (exp);
-
-		gtk_window_set_resizable (GTK_WINDOW (dlg), TRUE);
+				    label, TRUE, TRUE, 0);
+		gtk_widget_show (label);
 	}
 
 	return dlg;
 }
 
-/**
+/*
  * _gdaui_utility_display_error_with_keep_or_discard_choice
  * @form: a #GdauiDataProxy
  * @filled_error: a #GError containing the error to display
@@ -483,7 +460,7 @@ _gdaui_utility_display_error_with_keep_or_discard_choice (GdauiDataProxy *form,
 	return (res == GTK_RESPONSE_REJECT) ? TRUE : FALSE;
 }
 
-/**
+/*
  * _gdaui_utility_display_error
  * @form: a #GdauiDataProxy
  * @filled_error: a #GError containing the error to display
@@ -506,8 +483,8 @@ _gdaui_utility_display_error (GdauiDataProxy *form, gboolean can_discard, GError
 
 }
 
-/**
- * gnome_db_show_error
+/*
+ * _gdaui_utility_show_error
  * @format:
  * @...:
  *
@@ -604,7 +581,7 @@ _gdaui_setup_right_click_selection_on_treeview (GtkTreeView *tview)
 }
 
 static gboolean
-tree_view_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, gpointer unuseddata)
+tree_view_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, G_GNUC_UNUSED gpointer data)
 {
 	GtkTreeView *tree_view;
 	GtkTreeSelection *selection;
