@@ -1,25 +1,33 @@
 /*
- * Copyright (C) 1998 - 2011 The GNOME Foundation.
+ * Copyright (C) 2001 Cleber Rodrigues <cleberrrjr@bol.com.br>
+ * Copyright (C) 2001 - 2005 Rodrigo Moya <rodrigo@gnome-db.org>
+ * Copyright (C) 2002 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ * Copyright (C) 2003 Laurent Sansonetti <lrz@gnome.org>
+ * Copyright (C) 2004 Szalai Ferenc <szferi@einstein.ki.iif.hu>
+ * Copyright (C) 2005 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2005 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 lvaro Pea <alvaropg@telefonica.net>
+ * Copyright (C) 2006 - 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2008 Phil Longstaff <plongstaff@rogers.com>
+ * Copyright (C) 2008 Przemysław Grzegorczyk <pgrzegorczyk@gmail.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
+ * Copyright (C) 2011 Daniel Espinosa <esodan@gmail.com>
  *
- * AUTHORS:
- *	Rodrigo Moya <rodrigo@gnome-db.org>
- *      Vivien Malerba <malerba@gnome-db.org>
- *      Daniel Espinosa <esodan@gmail.com>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -182,7 +190,7 @@ gda_data_model_array_init (GdaDataModelArray *model, G_GNUC_UNUSED GdaDataModelA
 	/* allocate internal structure */
 	model->priv = g_new0 (GdaDataModelArrayPrivate, 1);
 	model->priv->notify_changes = TRUE;
-        model->priv->column_spec = g_hash_table_new (g_direct_hash, g_direct_equal);
+        model->priv->column_spec = g_hash_table_new_full (g_int_hash, g_int_equal, g_free, NULL);
         model->priv->read_only = FALSE;
 	model->priv->number_of_columns = 0;
 	model->priv->rows = g_array_new (FALSE, FALSE, sizeof (GdaRow *));
@@ -198,7 +206,6 @@ hash_free_column (G_GNUC_UNUSED gpointer key, GdaColumn *column, GdaDataModelArr
         g_object_unref (column);
 }
 
-
 static void
 gda_data_model_array_finalize (GObject *object)
 {
@@ -320,7 +327,7 @@ gda_data_model_array_new_with_g_types (gint cols, ...)
 /**
  * gda_data_model_array_new_with_g_types_v:
  * @cols: number of columns for rows in this data model.
- * @types: array of types of the columns of the model to create as #GType, as many as indicated by @cols
+ * @types: (array): array of types of the columns of the model to create as #GType, as many as indicated by @cols
  *
  * Creates a new #GdaDataModel object with the column types as
  * specified.
@@ -328,6 +335,8 @@ gda_data_model_array_new_with_g_types (gint cols, ...)
  * Returns: (transfer full): a pointer to the newly created #GdaDataModel.
  *
  * Since: 4.2.6
+ *
+ * Rename to: gda_data_model_array_new_with_g_types
  */
 GdaDataModel *
 gda_data_model_array_new_with_g_types_v (gint cols, GType *types)
@@ -491,15 +500,19 @@ gda_data_model_array_describe_column (GdaDataModel *model, gint col)
                 return NULL;
         }
 
-        column = g_hash_table_lookup (((GdaDataModelArray*) model)->priv->column_spec,
-                                      GINT_TO_POINTER (col));
+	gint tmp;
+	tmp = col;
+        column = g_hash_table_lookup (((GdaDataModelArray*) model)->priv->column_spec, &tmp);
         if (!column) {
                 column = gda_column_new ();
                 g_signal_connect (G_OBJECT (column), "g-type-changed",
                                   G_CALLBACK (column_g_type_changed_cb), model);
                 gda_column_set_position (column, col);
-                g_hash_table_insert (((GdaDataModelArray*) model)->priv->column_spec,
-                                     GINT_TO_POINTER (col), column);
+
+		gint *ptr;
+		ptr = g_new (gint, 1);
+		*ptr = col;
+                g_hash_table_insert (((GdaDataModelArray*) model)->priv->column_spec, ptr, column);
         }
 
         return column;
