@@ -1,21 +1,25 @@
-/* gda-holder.c
+/*
+ * Copyright (C) 2008 Massimo Cora <maxcvs@email.it>
+ * Copyright (C) 2008 Murray Cumming <murrayc@src.gnome.org>
+ * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  *
- * Copyright (C) 2003 - 2010 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -51,7 +55,15 @@ static void gda_holder_get_property (GObject *object,
 				     GValue *value,
 				     GParamSpec *pspec);
 
-static void full_bind_changed_cb (GdaHolder *alias_of, GdaHolder *holder);
+/* GdaLockable interface */
+static void                 gda_holder_lockable_init (GdaLockableIface *iface);
+static void                 gda_holder_lock      (GdaLockable *lockable);
+static gboolean             gda_holder_trylock   (GdaLockable *lockable);
+static void                 gda_holder_unlock    (GdaLockable *lockable);
+
+
+static void bound_holder_changed_cb (GdaHolder *alias_of, GdaHolder *holder);
+static void full_bound_holder_changed_cb (GdaHolder *alias_of, GdaHolder *holder);
 static void gda_holder_set_full_bind (GdaHolder *holder, GdaHolder *alias_of);
 
 /* get a pointer to the parents to be able to call their destructor */
@@ -94,9 +106,10 @@ struct _GdaHolderPrivate
 	GType            g_type;
 	GdaHolder       *full_bind;     /* FULL bind to holder */
 	GdaHolder       *simple_bind;  /* SIMPLE bind to holder */
-	gulong           simple_bind_notify_signal_id;
+	gulong           simple_bind_type_changed_id;
 	
 	gboolean         invalid_forced;
+	GError          *invalid_error;
 	gboolean         valid;
 	gboolean         is_freeable;
 
@@ -107,6 +120,8 @@ struct _GdaHolderPrivate
 
 	GdaDataModel    *source_model;
 	gint             source_col;
+
+	GdaMutex        *mutex;
 };
 
 /* module error */
@@ -134,12 +149,21 @@ gda_holder_get_type (void)
 			NULL,
 			sizeof (GdaHolder),
 			0,
-			(GInstanceInitFunc) gda_holder_init
+			(GInstanceInitFunc) gda_holder_init,
+			0
 		};
+
+		static GInterfaceInfo lockable_info = {
+                        (GInterfaceInitFunc) gda_holder_lockable_init,
+			NULL,
+                        NULL
+                };
 		
 		g_static_mutex_lock (&registering);
-		if (type == 0)
+		if (type == 0) {
 			type = g_type_register_static (G_TYPE_OBJECT, "GdaHolder", &info, 0);
+			g_type_add_interface_static (type, GDA_TYPE_LOCKABLE, &lockable_info);
+		}
 		g_static_mutex_unlock (&registering);
 	}
 
@@ -147,10 +171,10 @@ gda_holder_get_type (void)
 }
 
 static gboolean
-validate_change_accumulator (GSignalInvocationHint *ihint,
+validate_change_accumulator (G_GNUC_UNUSED GSignalInvocationHint *ihint,
 			   GValue *return_accu,
 			   const GValue *handler_return,
-			   gpointer data)
+			   G_GNUC_UNUSED gpointer data)
 {
 	GError *error;
 
@@ -161,13 +185,14 @@ validate_change_accumulator (GSignalInvocationHint *ihint,
 }
 
 static GError *
-m_validate_change (GdaHolder *holder, const GValue *new_value)
+m_validate_change (G_GNUC_UNUSED GdaHolder *holder, G_GNUC_UNUSED const GValue *new_value)
 {
 	return NULL;
 }
 
 static void
-holder_attribute_set_cb (GObject *obj, const gchar *att_name, const GValue *value, gpointer data)
+holder_attribute_set_cb (GObject *obj, const gchar *att_name, const GValue *value,
+			 G_GNUC_UNUSED gpointer data)
 {
 	g_signal_emit (obj, gda_holder_signals[ATT_CHANGED], 0, att_name, value);
 }
@@ -230,7 +255,7 @@ gda_holder_class_init (GdaHolderClass *class)
 	 * Gets emitted when @holder is going to change its value. One can connect to
 	 * this signal to control which values @holder can have (for example to implement some business rules)
 	 *
-	 * Return value: NULL if @holder is allowed to change its value to @new_value, or a #GError
+	 * Returns: NULL if @holder is allowed to change its value to @new_value, or a #GError
 	 * otherwise.
 	 */
 	gda_holder_signals[VALIDATE_CHANGE] =
@@ -294,6 +319,14 @@ gda_holder_class_init (GdaHolderClass *class)
 	gda_holder_attributes_manager = gda_attributes_manager_new (TRUE, holder_attribute_set_cb, NULL);
 }
 
+static void
+gda_holder_lockable_init (GdaLockableIface *iface)
+{
+	iface->i_lock = gda_holder_lock;
+	iface->i_trylock = gda_holder_trylock;
+	iface->i_unlock = gda_holder_unlock;
+}
+
 static void
 gda_holder_init (GdaHolder *holder)
 {
@@ -304,9 +337,10 @@ gda_holder_init (GdaHolder *holder)
 	holder->priv->g_type = G_TYPE_INVALID;
 	holder->priv->full_bind = NULL;
 	holder->priv->simple_bind = NULL;
-	holder->priv->simple_bind_notify_signal_id = 0;
+	holder->priv->simple_bind_type_changed_id = 0;
 
 	holder->priv->invalid_forced = FALSE;
+	holder->priv->invalid_error = NULL;
 	holder->priv->valid = TRUE;
 	holder->priv->default_forced = FALSE;
 	holder->priv->is_freeable = TRUE;
@@ -316,6 +350,8 @@ gda_holder_init (GdaHolder *holder)
 	holder->priv->not_null = FALSE;
 	holder->priv->source_model = NULL;
 	holder->priv->source_col = 0;
+
+	holder->priv->mutex = gda_mutex_new ();
 }
 
 /**
@@ -341,7 +377,7 @@ gda_holder_new (GType type)
  * Note1: if @orig is set with a static value (see gda_holder_take_static_value()) 
  * its copy will have a fresh new allocated GValue, so that user should free it when done.
  *
- * Returns: a new #GdaHolder object
+ * Returns: (transfer full): a new #GdaHolder object
  */
 GdaHolder *
 gda_holder_copy (GdaHolder *orig)
@@ -353,6 +389,7 @@ gda_holder_copy (GdaHolder *orig)
 	g_return_val_if_fail (orig && GDA_IS_HOLDER (orig), NULL);
 	g_return_val_if_fail (orig->priv, NULL);
 
+	gda_holder_lock ((GdaLockable*) orig);
 	obj = g_object_new (GDA_TYPE_HOLDER, "g-type", orig->priv->g_type, NULL);
 	holder = GDA_HOLDER (obj);
 
@@ -373,6 +410,8 @@ gda_holder_copy (GdaHolder *orig)
 	if (allok) {
 		/* direct settings */
 		holder->priv->invalid_forced = orig->priv->invalid_forced;
+		if (orig->priv->invalid_error)
+			holder->priv->invalid_error = g_error_copy (orig->priv->invalid_error);
 		holder->priv->valid = orig->priv->valid;
 		holder->priv->is_freeable = TRUE;
 		holder->priv->default_forced = orig->priv->default_forced;	
@@ -389,19 +428,21 @@ gda_holder_copy (GdaHolder *orig)
 		gda_value_free (att_value);
 
 
+		gda_holder_unlock ((GdaLockable*) orig);
 		return holder;
 	}
 	else {
 		g_warning ("Internal error: could not copy GdaHolder (please report a bug).");
 		g_object_unref (holder);
+		gda_holder_unlock ((GdaLockable*) orig);
 		return NULL;
 	}
 }
 
 /**
- * gda_holder_new_inline:
+ * gda_holder_new_inline: (skip)
  * @type: a valid GLib type
- * @id: the id of the holder to create, or %NULL
+ * @id: (allow-none): the id of the holder to create, or %NULL
  * @...: value to set
  *
  * Creates a new #GdaHolder object with an ID set to @id, of type @type, 
@@ -524,6 +565,11 @@ gda_holder_dispose (GObject *object)
 			gda_value_free (holder->priv->default_value);
 			holder->priv->default_value = NULL;
 		}
+
+		if (holder->priv->invalid_error) {
+			g_error_free (holder->priv->invalid_error);
+			holder->priv->invalid_error = NULL;
+		}
 	}
 
 	/* parent class */
@@ -542,6 +588,8 @@ gda_holder_finalize (GObject   * object)
 	if (holder->priv) {
 		g_free (holder->priv->id);
 
+		gda_mutex_free (holder->priv->mutex);
+
 		g_free (holder->priv);
 		holder->priv = NULL;
 	}
@@ -599,11 +647,11 @@ gda_holder_set_property (GObject *object,
 			break;
 		}
 		case PROP_SIMPLE_BIND:
-			if (!gda_holder_set_bind (holder, GDA_HOLDER (g_value_get_object (value)), NULL))
+			if (!gda_holder_set_bind (holder, (GdaHolder*) g_value_get_object (value), NULL))
 				g_warning ("Could not set the 'simple-bind' property");
 			break;
 		case PROP_FULL_BIND:
-			gda_holder_set_full_bind (holder, GDA_HOLDER (g_value_get_object (value)));
+			gda_holder_set_full_bind (holder, (GdaHolder*) g_value_get_object (value));
 			break;
 		case PROP_SOURCE_MODEL: {
 			GdaDataModel* ptr = g_value_get_object (value);
@@ -614,6 +662,9 @@ gda_holder_set_property (GObject *object,
 		case PROP_SOURCE_COLUMN:
 			holder->priv->source_col = g_value_get_int (value);
 			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
@@ -654,17 +705,20 @@ gda_holder_get_property (GObject *object,
 			g_value_set_boolean (value, gda_holder_get_not_null (holder));
 			break;
 		case PROP_SIMPLE_BIND:
-			g_value_set_object (value, G_OBJECT (holder->priv->simple_bind));
+			g_value_set_object (value, (GObject*) holder->priv->simple_bind);
 			break;
 		case PROP_FULL_BIND:
-			g_value_set_object (value, G_OBJECT (holder->priv->full_bind));
+			g_value_set_object (value, (GObject*) holder->priv->full_bind);
 			break;
 		case PROP_SOURCE_MODEL:
-			g_value_set_object (value, G_OBJECT (holder->priv->source_model));
+			g_value_set_object (value, (GObject*) holder->priv->source_model);
 			break;
 		case PROP_SOURCE_COLUMN:
 			g_value_set_int (value, holder->priv->source_col);
 			break;	
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+			break;
 		}
 	}
 }
@@ -675,7 +729,7 @@ gda_holder_get_property (GObject *object,
  * 
  * Get @holder's type
  *
- * Returns: (tranfer none): the data type
+ * Returns: (transfer none): the data type
  */
 GType
 gda_holder_get_g_type (GdaHolder *holder)
@@ -713,6 +767,8 @@ gda_holder_get_id (GdaHolder *holder)
  *
  * If @holder is set to NULL, then the returned value is a #GDA_TYPE_NULL GValue.
  *
+ * If @holder is invalid, then the returned value is %NULL.
+ *
  * Returns: the value, or %NULL
  */
 const GValue *
@@ -721,22 +777,26 @@ gda_holder_get_value (GdaHolder *holder)
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), NULL);
 	g_return_val_if_fail (holder->priv, NULL);
 
-	if (!holder->priv->full_bind) {
-		/* return default value if possible */
-		if (holder->priv->default_forced) {
-			g_assert (holder->priv->default_value);
-			if (G_VALUE_TYPE (holder->priv->default_value) == holder->priv->g_type) 
-				return holder->priv->default_value;
-			else
-				return NULL;
-		}
-
-		if (!holder->priv->value)
-			holder->priv->value = gda_value_new_null ();
-		return holder->priv->value;
-	}
-	else 
+	if (holder->priv->full_bind)
 		return gda_holder_get_value (holder->priv->full_bind);
+	else {
+		if (holder->priv->valid) {
+			/* return default value if possible */
+			if (holder->priv->default_forced) {
+				g_assert (holder->priv->default_value);
+				if (G_VALUE_TYPE (holder->priv->default_value) == holder->priv->g_type) 
+					return holder->priv->default_value;
+				else
+					return NULL;
+			}
+			
+			if (!holder->priv->value)
+				holder->priv->value = gda_value_new_null ();
+			return holder->priv->value;
+		}
+		else
+			return NULL;
+	}
 }
 
 /**
@@ -757,16 +817,20 @@ gda_holder_get_value_str (GdaHolder *holder, GdaDataHandler *dh)
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), NULL);
 	g_return_val_if_fail (holder->priv, NULL);
 
+	gda_holder_lock ((GdaLockable*) holder);
 	current_val = gda_holder_get_value (holder);
-        if (!current_val || __gda_value_is_null (current_val))
+        if (!current_val || __gda_value_is_null (current_val)) {
+		gda_holder_unlock ((GdaLockable*) holder);
                 return NULL;
+	}
         else {
+		gchar *retval = NULL;
                 if (!dh)
-			dh = gda_get_default_handler (holder->priv->g_type);
+			dh = gda_data_handler_get_default (holder->priv->g_type);
 		if (dh)
-                        return gda_data_handler_get_str_from_value (dh, current_val);
-                else
-                        return NULL;
+                        retval = gda_data_handler_get_str_from_value (dh, current_val);
+		gda_holder_unlock ((GdaLockable*) holder);
+		return retval;
         }
 }
 
@@ -835,20 +899,22 @@ gda_holder_set_value_str (GdaHolder *holder, GdaDataHandler *dh, const gchar *va
                 return gda_holder_set_value (holder, NULL, error);
     else {
 		GValue *gdaval = NULL;
+		gboolean retval = FALSE;
 
+		gda_holder_lock ((GdaLockable*) holder);
 		if (!dh)
-			dh = gda_get_default_handler (holder->priv->g_type);
+			dh = gda_data_handler_get_default (holder->priv->g_type);
 		if (dh)
 			gdaval = gda_data_handler_get_value_from_str (dh, value, holder->priv->g_type);
 
 		if (gdaval)
-			return real_gda_holder_set_value (holder, gdaval, FALSE, error);
-        else {
+			retval = real_gda_holder_set_value (holder, gdaval, FALSE, error);
+		else
 			g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_STRING_CONVERSION_ERROR,
 				     _("Unable to convert string to '%s' type"), 
 				     gda_g_type_to_string (holder->priv->g_type));
-        	return FALSE;
-		}
+		gda_holder_unlock ((GdaLockable*) holder);
+		return retval;
 	}
 }
 
@@ -896,15 +962,18 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 	gboolean newvalid;
 	const GValue *current_val;
 	gboolean newnull;
+	gboolean was_valid;
 #define DEBUG_HOLDER
 #undef DEBUG_HOLDER
 
-	gboolean was_valid = gda_holder_is_valid (holder);
+	gda_holder_lock ((GdaLockable*) holder);
+	was_valid = gda_holder_is_valid (holder);
 
 	/* if the value has been set with gda_holder_take_static_value () you'll be able
 	 * to change the value only with another call to real_gda_holder_set_value 
 	 */
 	if (!holder->priv->is_freeable) {
+		gda_holder_unlock ((GdaLockable*) holder);
 		g_warning (_("Can't use this method to set value because there is already a static value"));
 		return FALSE;
 	}
@@ -957,7 +1026,12 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 		if (!do_copy && value)
 			gda_value_free (value);
 		holder->priv->invalid_forced = FALSE;
+		if (holder->priv->invalid_error) {
+			g_error_free (holder->priv->invalid_error);
+			holder->priv->invalid_error = NULL;
+		}
 		holder->priv->valid = newvalid;
+		gda_holder_unlock ((GdaLockable*) holder);
 		return TRUE;
 	}
 
@@ -973,11 +1047,16 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 		g_propagate_error (error, lerror);
 		if (!do_copy) 
 			gda_value_free (value);
+		gda_holder_unlock ((GdaLockable*) holder);
 		return FALSE;
 	}
 
 	/* new valid status */
 	holder->priv->invalid_forced = FALSE;
+	if (holder->priv->invalid_error) {
+		g_error_free (holder->priv->invalid_error);
+		holder->priv->invalid_error = NULL;
+	}
 	holder->priv->valid = newvalid;
 	/* we're setting a non-static value, so be sure to flag is as freeable */
 	holder->priv->is_freeable = TRUE;
@@ -1002,6 +1081,7 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 		g_print ("Holder %p is alias of holder %p => propagating changes to holder %p\n",
 			 holder, holder->priv->full_bind, holder->priv->full_bind);
 #endif
+		gda_holder_unlock ((GdaLockable*) holder);
 		return real_gda_holder_set_value (holder->priv->full_bind, value, do_copy, error);
 	}
 	else {
@@ -1024,6 +1104,7 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 		g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
 	}
 
+	gda_holder_unlock ((GdaLockable*) holder);
 	return newvalid;
 }
 
@@ -1087,6 +1168,10 @@ real_gda_holder_set_const_value (GdaHolder *holder, const GValue *value,
 	/* end of procedure if the value has not been changed, after calculating the holder's validity */
 	if (!changed) {
 		holder->priv->invalid_forced = FALSE;
+		if (holder->priv->invalid_error) {
+			g_error_free (holder->priv->invalid_error);
+			holder->priv->invalid_error = NULL;
+		}
 		holder->priv->valid = newvalid;
 #ifdef DEBUG_HOLDER		
 		g_print ("Holder is not changed");
@@ -1109,6 +1194,10 @@ real_gda_holder_set_const_value (GdaHolder *holder, const GValue *value,
 
 	/* new valid status */
 	holder->priv->invalid_forced = FALSE;
+	if (holder->priv->invalid_error) {
+		g_error_free (holder->priv->invalid_error);
+		holder->priv->invalid_error = NULL;
+	}
 	holder->priv->valid = newvalid;
 	/* we're setting a static value, so be sure to flag is as unfreeable */
 	holder->priv->is_freeable = FALSE;
@@ -1192,10 +1281,15 @@ GValue *
 gda_holder_take_static_value (GdaHolder *holder, const GValue *value, gboolean *value_changed,
 			      GError **error)
 {
+	GValue *retvalue;
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
 	g_return_val_if_fail (holder->priv, FALSE);
 
-	return real_gda_holder_set_const_value (holder, value, value_changed, error);
+	gda_holder_lock ((GdaLockable*) holder);
+	retvalue = real_gda_holder_set_const_value (holder, value, value_changed, error);
+	gda_holder_unlock ((GdaLockable*) holder);
+
+	return retvalue;
 }
 
 /**
@@ -1209,6 +1303,25 @@ gda_holder_take_static_value (GdaHolder *holder, const GValue *value, gboolean *
  */
 void
 gda_holder_force_invalid (GdaHolder *holder)
+{
+	g_return_if_fail (GDA_IS_HOLDER (holder));
+	gda_holder_force_invalid_e (holder, NULL);
+}
+
+/**
+ * gda_holder_force_invalid_e:
+ * @holder: a #GdaHolder object
+ * @error: (allow-none) (transfer full): a #GError explaining why @holder is declared invalid, or %NULL
+ *
+ * Forces a holder to be invalid; to set it valid again, a new value must be assigned
+ * to it using gda_holder_set_value() or gda_holder_take_value().
+ *
+ * @holder's value is set to %NULL.
+ *
+ * Since: 4.2.10
+ */
+void
+gda_holder_force_invalid_e (GdaHolder *holder, GError *error)
 {
 	g_return_if_fail (GDA_IS_HOLDER (holder));
 	g_return_if_fail (holder->priv);
@@ -1217,12 +1330,18 @@ gda_holder_force_invalid (GdaHolder *holder)
 	g_print ("Holder %p (%s): declare invalid\n", holder, holder->priv->id);
 #endif
 
-	if (holder->priv->invalid_forced)
+	gda_holder_lock ((GdaLockable*) holder);
+	if (holder->priv->invalid_error)
+		g_error_free (holder->priv->invalid_error);
+	holder->priv->invalid_error = error;
+
+	if (holder->priv->invalid_forced) {
+		gda_holder_unlock ((GdaLockable*) holder);
 		return;
+	}
 
 	holder->priv->invalid_forced = TRUE;
 	holder->priv->valid = FALSE;
-	
 	if (holder->priv->value) {
 		if (holder->priv->is_freeable)
 			gda_value_free (holder->priv->value);
@@ -1234,9 +1353,9 @@ gda_holder_force_invalid (GdaHolder *holder)
 		gda_holder_force_invalid (holder->priv->full_bind);
 	else 
 		g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
+	gda_holder_unlock ((GdaLockable*) holder);
 }
 
-
 /**
  * gda_holder_is_valid:
  * @holder: a #GdaHolder object
@@ -1248,20 +1367,45 @@ gda_holder_force_invalid (GdaHolder *holder)
 gboolean
 gda_holder_is_valid (GdaHolder *holder)
 {
+	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
+	return gda_holder_is_valid_e (holder, NULL);
+}
+
+/**
+ * gda_holder_is_valid_e:
+ * @holder: a #GdaHolder object
+ * @error: (allow-none): a place to store invalid error, or %NULL
+ *
+ * Get the validity of @holder (that is, of the value held by @holder)
+ *
+ * Returns: TRUE if @holder's value can safely be used
+ *
+ * Since: 4.2.10
+ */
+gboolean
+gda_holder_is_valid_e (GdaHolder *holder, GError **error)
+{
+	gboolean retval;
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
 	g_return_val_if_fail (holder->priv, FALSE);
 
+	gda_holder_lock ((GdaLockable*) holder);
 	if (holder->priv->full_bind)
-		return gda_holder_is_valid (holder->priv->full_bind);
+		retval = gda_holder_is_valid_e (holder->priv->full_bind, error);
 	else {
 		if (holder->priv->invalid_forced) 
-			return FALSE;
-
-		if (holder->priv->default_forced) 
-			return holder->priv->default_value ? TRUE : FALSE;
-		else 
-			return holder->priv->valid;
+			retval = FALSE;
+		else {
+			if (holder->priv->default_forced) 
+				retval = holder->priv->default_value ? TRUE : FALSE;
+			else 
+				retval = holder->priv->valid;
+		}
+		if (!retval && holder->priv->invalid_error)
+			g_propagate_error (error,  g_error_copy (holder->priv->invalid_error));
 	}
+	gda_holder_unlock ((GdaLockable*) holder);
+	return retval;
 }
 
 /**
@@ -1278,14 +1422,24 @@ gda_holder_set_value_to_default (GdaHolder *holder)
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
 	g_return_val_if_fail (holder->priv, FALSE);
 
-	if (holder->priv->default_forced)
+	gda_holder_lock ((GdaLockable*) holder);
+	if (holder->priv->default_forced) {
+		gda_holder_unlock ((GdaLockable*) holder);
 		return TRUE;
+	}
 
-	if (!holder->priv->default_value)
+	if (!holder->priv->default_value) {
+		gda_holder_unlock ((GdaLockable*) holder);
 		return FALSE;
+	}
 	else {
 		holder->priv->default_forced = TRUE;
 		holder->priv->invalid_forced = FALSE;
+		if (holder->priv->invalid_error) {
+			g_error_free (holder->priv->invalid_error);
+			holder->priv->invalid_error = NULL;
+		}
+
 		if (holder->priv->value) {
 			if (holder->priv->is_freeable)
 				gda_value_free (holder->priv->value);
@@ -1299,6 +1453,7 @@ gda_holder_set_value_to_default (GdaHolder *holder)
 	gda_value_free (att_value);
 	g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
 
+	gda_holder_unlock ((GdaLockable*) holder);
 	return TRUE;
 }
 
@@ -1356,6 +1511,7 @@ gda_holder_set_default_value (GdaHolder *holder, const GValue *value)
 	g_return_if_fail (GDA_IS_HOLDER (holder));
 	g_return_if_fail (holder->priv);
 
+	gda_holder_lock ((GdaLockable*) holder);
 	if (holder->priv->default_value) {
 		if (holder->priv->default_forced) {
 			gda_holder_take_value (holder, holder->priv->default_value, NULL);
@@ -1389,6 +1545,7 @@ gda_holder_set_default_value (GdaHolder *holder, const GValue *value)
 	gda_value_free (att_value);
 
 	/* don't emit the "changed" signal */
+	gda_holder_unlock ((GdaLockable*) holder);
 }
 
 /**
@@ -1427,7 +1584,7 @@ gda_holder_get_not_null (GdaHolder *holder)
 /**
  * gda_holder_set_source_model:
  * @holder: a #GdaHolder object
- * @model: a #GdaDataModel object or NULL
+ * @model: a #GdaDataModel object or %NULL
  * @col: the reference column in @model
  * @error: location to store error, or %NULL
  *
@@ -1435,6 +1592,9 @@ gda_holder_get_not_null (GdaHolder *holder)
  * contained in the @col column of the @model data model. Note that this is just a hint,
  * meaning this policy is not enforced by @holder's implementation.
  *
+ * If @model is %NULL, then the effect is to cancel ant previous call to gda_holder_set_source_model()
+ * where @model was not %NULL.
+ *
  * Returns: TRUE if no error occurred
  */
 gboolean
@@ -1443,27 +1603,47 @@ gda_holder_set_source_model (GdaHolder *holder, GdaDataModel *model,
 {
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
 	g_return_val_if_fail (holder->priv, FALSE);
+	if (model)
+		g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 
 	/* No check is done on the validity of @col or even its existance */
 	/* Note: for internal implementation if @col<0, then it's ignored */
 
-	if (holder->priv->source_model == model) {
-		if (col >= 0)
-			holder->priv->source_col = col;
+	gda_holder_lock ((GdaLockable*) holder);
+	if (model && (col >= 0)) {
+		GType htype, ctype;
+		GdaColumn *gcol;
+		htype = gda_holder_get_g_type (holder);
+		gcol = gda_data_model_describe_column (model, col);
+		if (gcol) {
+			ctype = gda_column_get_g_type (gcol);
+			if ((htype != GDA_TYPE_NULL) && (ctype != GDA_TYPE_NULL) &&
+			    (htype != ctype)) {
+				g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_TYPE_ERROR,
+					     _("GdaHolder has a gda type (%s) incompatible with "
+                                               "source column %d type (%s)"),
+                                             gda_g_type_to_string (htype),
+                                             col, gda_g_type_to_string (ctype));
+				gda_holder_unlock ((GdaLockable*) holder);
+				return FALSE;
+			}
+		}
 	}
-	else {
+
+	if (col >= 0)
+		holder->priv->source_col = col;
+
+	if (holder->priv->source_model != model) {
 		if (holder->priv->source_model) {
 			g_object_unref (holder->priv->source_model);
 			holder->priv->source_model = NULL;
 		}
 		
-		if (col >= 0)
-			holder->priv->source_col = col;
-		
-		if (model) {
-			holder->priv->source_model = model;
+		holder->priv->source_model = model;
+		if (model)
 			g_object_ref (model);
-		}
+		else
+			holder->priv->source_col = 0;
 	}
 
 #ifdef GDA_DEBUG_signal
@@ -1473,6 +1653,8 @@ gda_holder_set_source_model (GdaHolder *holder, GdaDataModel *model,
 #ifdef GDA_DEBUG_signal
         g_print ("<< 'SOURCE_CHANGED' from %p\n", holder);
 #endif
+
+	gda_holder_unlock ((GdaLockable*) holder);
 	return TRUE;
 }
 
@@ -1494,13 +1676,16 @@ gda_holder_set_source_model (GdaHolder *holder, GdaDataModel *model,
 GdaDataModel *
 gda_holder_get_source_model (GdaHolder *holder, gint *col)
 {
+	GdaDataModel *model;
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), FALSE);
 	g_return_val_if_fail (holder->priv, FALSE);
-	
+
+	gda_holder_lock ((GdaLockable*) holder);
 	if (col)
 		*col = holder->priv->source_col;
-
-	return holder->priv->source_model;
+	model = holder->priv->source_model;
+	gda_holder_unlock ((GdaLockable*) holder);
+	return model;
 }
 
 /*
@@ -1508,11 +1693,14 @@ gda_holder_get_source_model (GdaHolder *holder, gint *col)
  * gda_holder_set_bind() was called, and it makes sure @holder's GType is the same as @holder->priv->simple_bind's
  */
 static void
-bind_to_notify_cb (GdaHolder *bind_to, GParamSpec *pspec, GdaHolder *holder)
+bind_to_notify_cb (GdaHolder *bind_to, G_GNUC_UNUSED GParamSpec *pspec, GdaHolder *holder)
 {
+	gda_holder_lock ((GdaLockable*) holder);
+	gda_holder_lock ((GdaLockable*) bind_to);
+
 	g_signal_handler_disconnect (holder->priv->simple_bind,
-				     holder->priv->simple_bind_notify_signal_id);
-	holder->priv->simple_bind_notify_signal_id = 0;
+				     holder->priv->simple_bind_type_changed_id);
+	holder->priv->simple_bind_type_changed_id = 0;
 	if (holder->priv->g_type == GDA_TYPE_NULL) {
 		holder->priv->g_type = bind_to->priv->g_type;
 		g_object_notify ((GObject*) holder, "g-type");
@@ -1524,6 +1712,9 @@ bind_to_notify_cb (GdaHolder *bind_to, GParamSpec *pspec, GdaHolder *holder)
 			   gda_holder_get_id (holder), gda_holder_get_id (bind_to));
 		gda_holder_set_bind (holder, NULL, NULL);
 	}
+
+	gda_holder_unlock ((GdaLockable*) holder);
+	gda_holder_unlock ((GdaLockable*) bind_to);
 }
 
 /**
@@ -1552,8 +1743,11 @@ gda_holder_set_bind (GdaHolder *holder, GdaHolder *bind_to, GError **error)
 	g_return_val_if_fail (holder->priv, FALSE);
 	g_return_val_if_fail (holder != bind_to, FALSE);
 
-	if (holder->priv->simple_bind == bind_to)
+	gda_holder_lock ((GdaLockable*) holder);
+	if (holder->priv->simple_bind == bind_to) {
+		gda_holder_unlock ((GdaLockable*) holder);
 		return TRUE;
+	}
 
 	/* get a copy of the current values of @holder and @bind_to */
 	if (bind_to) {
@@ -1565,6 +1759,7 @@ gda_holder_set_bind (GdaHolder *holder, GdaHolder *bind_to, GError **error)
 		    (holder->priv->g_type != bind_to->priv->g_type)) {
 			g_set_error (error, GDA_HOLDER_ERROR, GDA_HOLDER_VALUE_TYPE_ERROR,
 				     "%s", _("Cannot bind holders if their type is not the same"));
+			gda_holder_unlock ((GdaLockable*) holder);
 			return FALSE;
 		}
 		value2 = gda_holder_get_value (bind_to);
@@ -1576,38 +1771,42 @@ gda_holder_set_bind (GdaHolder *holder, GdaHolder *bind_to, GError **error)
 
 	/* get rid of the old alias */
 	if (holder->priv->simple_bind) {
-		g_signal_handlers_disconnect_by_func (G_OBJECT (holder->priv->simple_bind),
-						      G_CALLBACK (full_bind_changed_cb), holder);
-		if (holder->priv->simple_bind_notify_signal_id) {
+		g_signal_handlers_disconnect_by_func (holder->priv->simple_bind,
+						      G_CALLBACK (bound_holder_changed_cb), holder);
+
+		if (holder->priv->simple_bind_type_changed_id) {
 			g_signal_handler_disconnect (holder->priv->simple_bind,
-						     holder->priv->simple_bind_notify_signal_id);
-			holder->priv->simple_bind_notify_signal_id = 0;
+						     holder->priv->simple_bind_type_changed_id);
+			holder->priv->simple_bind_type_changed_id = 0;
 		}
 		g_object_unref (holder->priv->simple_bind);
 		holder->priv->simple_bind = NULL;
 	}
 
 	/* setting the new alias or reseting the value if there is no new alias */
+	gboolean retval;
 	if (bind_to) {
-		holder->priv->simple_bind = bind_to;
-		g_object_ref (holder->priv->simple_bind);
-		g_signal_connect (G_OBJECT (holder->priv->simple_bind), "changed",
-				  G_CALLBACK (full_bind_changed_cb), holder);
+		holder->priv->simple_bind = g_object_ref (bind_to);
+		g_signal_connect (holder->priv->simple_bind, "changed",
+				  G_CALLBACK (bound_holder_changed_cb), holder);
 
 		if (bind_to->priv->g_type == GDA_TYPE_NULL)
-			holder->priv->simple_bind_notify_signal_id = g_signal_connect (bind_to, "notify::g-type",
-										       G_CALLBACK (bind_to_notify_cb),
-										       holder);
+			holder->priv->simple_bind_type_changed_id = g_signal_connect (bind_to, "notify::g-type",
+										      G_CALLBACK (bind_to_notify_cb),
+										      holder);
 		else if (holder->priv->g_type == GDA_TYPE_NULL)
 			g_object_set ((GObject*) holder, "g-type", bind_to->priv->g_type , NULL);
 
 		/* if bind_to has a different value than holder, then we set holder to the new value */
 		if (value1)
 			gda_value_free (value1);
-		return gda_holder_set_value (holder, value2, error);
+		retval = gda_holder_set_value (holder, value2, error);
 	}
 	else
-		return gda_holder_take_value (holder, value1, error);
+		retval = gda_holder_take_value (holder, value1, error);
+
+	gda_holder_unlock ((GdaLockable*) holder);
+	return retval;
 }
 
 /*
@@ -1628,8 +1827,11 @@ gda_holder_set_full_bind (GdaHolder *holder, GdaHolder *alias_of)
 	g_return_if_fail (GDA_IS_HOLDER (holder));
 	g_return_if_fail (holder->priv);
 
-	if (holder->priv->full_bind == alias_of)
+	gda_holder_lock ((GdaLockable*) holder);
+	if (holder->priv->full_bind == alias_of) {
+		gda_holder_unlock ((GdaLockable*) holder);
 		return;
+	}
 
 	/* get a copy of the current values of @holder and @alias_of */
 	if (alias_of) {
@@ -1644,12 +1846,11 @@ gda_holder_set_full_bind (GdaHolder *holder, GdaHolder *alias_of)
 	cvalue = gda_holder_get_value (holder);
 	if (cvalue && !__gda_value_is_null ((GValue*)cvalue))
 		value1 = gda_value_copy ((GValue*)cvalue);
-		
 	
 	/* get rid of the old alias */
 	if (holder->priv->full_bind) {
-		g_signal_handlers_disconnect_by_func (G_OBJECT (holder->priv->full_bind),
-						      G_CALLBACK (full_bind_changed_cb), holder);
+		g_signal_handlers_disconnect_by_func (holder->priv->full_bind,
+						      G_CALLBACK (full_bound_holder_changed_cb), holder);
 		g_object_unref (holder->priv->full_bind);
 		holder->priv->full_bind = NULL;
 	}
@@ -1665,10 +1866,9 @@ gda_holder_set_full_bind (GdaHolder *holder, GdaHolder *alias_of)
 			holder->priv->value = NULL;
 		}
 
-		holder->priv->full_bind = alias_of;
-		g_object_ref (alias_of);
-		g_signal_connect (G_OBJECT (alias_of), "changed",
-				  G_CALLBACK (full_bind_changed_cb), holder);
+		holder->priv->full_bind = g_object_ref (alias_of);
+		g_signal_connect (holder->priv->full_bind, "changed",
+				  G_CALLBACK (full_bound_holder_changed_cb), holder);
 
 		/* if alias_of has a different value than holder, then we emit a CHANGED signal */
 		if (value1 && value2 &&
@@ -1693,24 +1893,40 @@ gda_holder_set_full_bind (GdaHolder *holder, GdaHolder *alias_of)
 
 	if (value1) gda_value_free (value1);
 	if (value2) gda_value_free (value2);
+	gda_holder_unlock ((GdaLockable*) holder);
 }
 
 static void
-full_bind_changed_cb (GdaHolder *alias_of, GdaHolder *holder)
+full_bound_holder_changed_cb (GdaHolder *alias_of, GdaHolder *holder)
 {
-	if (alias_of == holder->priv->simple_bind) {
-		const GValue *cvalue;
-		GError *lerror = NULL;
-		cvalue = gda_holder_get_value (alias_of);
-		if (! gda_holder_set_value (holder, cvalue, &lerror)) {
-			if (lerror && ((lerror->domain != GDA_HOLDER_ERROR) || (lerror->code != GDA_HOLDER_VALUE_NULL_ERROR)))
-				g_warning (_("Could not change GdaHolder to match value change in bound GdaHolder: %s"),
-					   lerror && lerror->message ? lerror->message : _("No detail"));
-			g_clear_error (&lerror);
-		}
+	gda_holder_lock ((GdaLockable*) holder);
+	gda_holder_lock ((GdaLockable*) alias_of);
+
+	g_assert (alias_of == holder->priv->full_bind);
+	g_signal_emit (holder, gda_holder_signals [CHANGED], 0);
+
+	gda_holder_unlock ((GdaLockable*) holder);
+	gda_holder_unlock ((GdaLockable*) alias_of);
+}
+
+static void
+bound_holder_changed_cb (GdaHolder *alias_of, GdaHolder *holder)
+{
+	gda_holder_lock ((GdaLockable*) holder);
+	gda_holder_lock ((GdaLockable*) alias_of);
+
+	g_assert (alias_of == holder->priv->simple_bind);
+	const GValue *cvalue;
+	GError *lerror = NULL;
+	cvalue = gda_holder_get_value (alias_of);
+	if (! gda_holder_set_value (holder, cvalue, &lerror)) {
+		if (lerror && ((lerror->domain != GDA_HOLDER_ERROR) || (lerror->code != GDA_HOLDER_VALUE_NULL_ERROR)))
+			g_warning (_("Could not change GdaHolder to match value change in bound GdaHolder: %s"),
+				   lerror && lerror->message ? lerror->message : _("No detail"));
+		g_clear_error (&lerror);
 	}
-	else
-		g_signal_emit (holder, gda_holder_signals [CHANGED], 0);
+	gda_holder_unlock ((GdaLockable*) holder);
+	gda_holder_unlock ((GdaLockable*) alias_of);
 }
 
 /**
@@ -1800,11 +2016,36 @@ gda_holder_set_attribute (GdaHolder *holder, const gchar *attribute, const GValu
 	const GValue *cvalue;
 	g_return_if_fail (GDA_IS_HOLDER (holder));
 
+	gda_holder_lock ((GdaLockable*) holder);
 	cvalue = gda_attributes_manager_get (gda_holder_attributes_manager, holder, attribute);
 	if ((value && cvalue && !gda_value_differ (cvalue, value)) ||
-	    (!value && !cvalue))
+	    (!value && !cvalue)) {
+		gda_holder_unlock ((GdaLockable*) holder);
 		return;
+	}
 
 	gda_attributes_manager_set_full (gda_holder_attributes_manager, holder, attribute, value, destroy);
 	//g_print ("GdaHolder %p ATTR '%s' set to '%s'\n", holder, attribute, gda_value_stringify (value)); 
+	gda_holder_unlock ((GdaLockable*) holder);
+}
+
+static void
+gda_holder_lock (GdaLockable *lockable)
+{
+	GdaHolder *holder = (GdaHolder *) lockable;
+	gda_mutex_lock (holder->priv->mutex);
+}
+
+static gboolean
+gda_holder_trylock (GdaLockable *lockable)
+{
+	GdaHolder *holder = (GdaHolder *) lockable;
+	return gda_mutex_trylock (holder->priv->mutex);
+}
+
+static void
+gda_holder_unlock (GdaLockable *lockable)
+{
+	GdaHolder *holder = (GdaHolder *) lockable;
+	gda_mutex_unlock (holder->priv->mutex);
 }
