@@ -402,7 +402,7 @@ gda_set_init (GdaSet *set)
 
 /**
  * gda_set_new:
- * @holders: (element-type GdaHolder) (transfer:none): a list of #GdaHolder objects
+ * @holders: (element-type Gda.Holder) (transfer none): a list of #GdaHolder objects
  *
  * Creates a new #GdaSet object, and populates it with the list given as argument.
  * The list can then be freed as it is copied. All the value holders in @holders are referenced counted
@@ -425,7 +425,7 @@ gda_set_new (GSList *holders)
 
 /**
  * gda_set_new_read_only:
- * @holders: (element-type GdaHolder) (transfer:none): a list of #GdaHolder objects
+ * @holders: (element-type Gda.Holder) (transfer none): a list of #GdaHolder objects
  *
  * Creates a new #GdaSet like gda_set_new(), but does not allow modifications to any of the #GdaHolder
  * object in @holders. This function is used for Libgda's database providers' implementation.
@@ -655,7 +655,7 @@ gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, .
 	else if (type == G_TYPE_GTYPE)
 		g_value_set_gtype (value, va_arg (ap, GType));	
 	else {
-		g_set_error (error, 0, 0,
+		g_set_error (error, GDA_SET_ERROR, GDA_SET_IMPLEMENTATION_ERROR,
 			     _("%s() does not handle values of type '%s'."),
 			     __FUNCTION__, g_type_name (type));
 		va_end (ap);
@@ -850,7 +850,8 @@ gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
 				errors = gda_data_model_import_get_errors (GDA_DATA_MODEL_IMPORT (model));
 				if (errors) {
 					GError *err = (GError *) errors->data;
-					g_set_error (error, 0, 0, "%s", err->message);
+					g_set_error (error, GDA_SET_ERROR, GDA_SET_XML_SPEC_ERROR,
+						     "%s", err->message);
 					g_object_unref (model);
 					model = NULL;
 					allok = FALSE;
