@@ -128,7 +128,7 @@ gda_sql_builder_class_init (GdaSqlBuilderClass *klass)
 	 */
 	g_object_class_install_property (object_class, PROP_TYPE,
 					 g_param_spec_enum ("stmt-type", NULL, "Statement Type",
-							    GDA_SQL_PARSER_TYPE_SQL_STATEMENT_TYPE,
+							    GDA_TYPE_SQL_STATEMENT_TYPE,
 							    GDA_SQL_STATEMENT_UNKNOWN,
 							    (G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY)));
 }
@@ -352,7 +352,7 @@ gda_sql_builder_get_statement (GdaSqlBuilder *builder, GError **error)
 }
 
 /**
- * gda_sql_builder_get_sql_statement:
+ * gda_sql_builder_get_sql_statement: (skip)
  * @builder: a #GdaSqlBuilder object
  *
  * Creates a new #GdaSqlStatement structure from @builder's contents.
@@ -519,12 +519,12 @@ gda_sql_builder_select_add_field (GdaSqlBuilder *builder, const gchar *field_nam
 	const GdaSqlBuilderId field_id = gda_sql_builder_add_id (builder, tmp);
 	if (alias && *alias)
 		gda_sql_builder_add_field_value_id (builder,
-					      field_id,
-					      gda_sql_builder_add_id (builder, alias));
+						    field_id,
+						    gda_sql_builder_add_id (builder, alias));
 	else
 		gda_sql_builder_add_field_value_id (builder,
-					      field_id,
-					      0);
+						    field_id,
+						    0);
 	if (tmp_is_allocated)
 		g_free (tmp);
 
@@ -799,70 +799,61 @@ gda_sql_builder_add_field_value_id (GdaSqlBuilder *builder, GdaSqlBuilderId fiel
 /**
  * gda_sql_builder_add_expr_value:
  * @builder: a #GdaSqlBuilder object
- * @dh: (allow-none): a #GdaDataHandler to use, or %NULL
+ * @dh: (allow-none): deprecated useless argument, just pass %NULL
  * @value: (allow-none): value to set the expression to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
  *
  * Defines an expression in @builder which may be reused to build other parts of a statement.
  *
- * The new expression will contain the value passed as the @value argument. It is possible to
- * customize how the value has to be interpreted by passing a specific #GdaDataHandler object as @dh.
+ * The new expression will contain the value passed as the @value argument.
+ *
+ * If @value's type is a string then it is possible to customize how the value has to be interpreted by passing a
+ * specific #GdaDataHandler object as @dh. This feature is very rarely used and the @dh argument should generally
+ * be %NULL.
  *
  * Returns: the ID of the new expression, or %0 if there was an error
  *
  * Since: 4.2
  */
 GdaSqlBuilderId
-gda_sql_builder_add_expr_value (GdaSqlBuilder *builder, GdaDataHandler *dh, const GValue *value)
+gda_sql_builder_add_expr_value (GdaSqlBuilder *builder, G_GNUC_UNUSED GdaDataHandler *dh, const GValue *value)
 {
 	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
 	g_return_val_if_fail (builder->priv->main_stmt, 0);
 
 	gchar *str;
-
+	GdaSqlExpr *expr;
+	expr = gda_sql_expr_new (NULL);
 	if (value && (G_VALUE_TYPE (value) != GDA_TYPE_NULL)) {
-		GType type = G_VALUE_TYPE (value);
-		if (!dh)
-			dh = gda_data_handler_get_default (type);
-		else {
-			if (! gda_data_handler_accepts_g_type (dh, type)) {
-				g_warning (_("Unhandled data type '%s'"), g_type_name (type));
-				return 0;
-			}
+		if (G_VALUE_TYPE (value) == G_TYPE_STRING) {
+			GdaDataHandler *ldh;
+			ldh = gda_data_handler_get_default (G_TYPE_STRING);
+			expr->value = gda_value_new (G_TYPE_STRING);
+			g_value_take_string (expr->value, gda_data_handler_get_sql_from_value (ldh, value));
 		}
-		if (!dh) {
-			g_warning (_("Unhandled data type '%s'"), g_type_name (type));
-			return 0;
-		}
-		str = gda_data_handler_get_sql_from_value (dh, value);
-	}
-	else
-		str = g_strdup ("NULL");
-
-	if (str) {
-		GdaSqlExpr *expr;
-		expr = gda_sql_expr_new (NULL);
-		expr->value = gda_value_new (G_TYPE_STRING);
-		g_value_take_string (expr->value, str);
-		return add_part (builder, (GdaSqlAnyPart *) expr);
+		else
+			expr->value = gda_value_copy (value);
 	}
 	else {
-		g_warning (_("Could not convert value to type '%s'"),
-			   g_type_name (G_VALUE_TYPE (value)));
-		return 0;
+		expr->value = gda_value_new (G_TYPE_STRING);
+		g_value_set_string (expr->value, "NULL");
 	}
+	return add_part (builder, (GdaSqlAnyPart *) expr);
 }
 
 /**
  * gda_sql_builder_add_expr: (skip)
  * @builder: a #GdaSqlBuilder object
- * @dh: (allow-none): a #GdaDataHandler to use, or %NULL
+ * @dh: (allow-none): deprecated useless argument, just pass %NULL
  * @type: the GType of the following argument
  * @...: value to set the expression to, of the type specified by @type
  *
  * Defines an expression in @builder which may be reused to build other parts of a statement.
  *
- * The new expression will contain the value passed as the @... argument. It is possible to
- * customize how the value has to be interpreted by passing a specific #GdaDataHandler object as @dh.
+ * The new expression will contain the value passed as the @... argument.
+ *
+ * If @type is G_TYPE_STRING then it is possible to customize how the value has to be interpreted by passing a
+ * specific #GdaDataHandler object as @dh. This feature is very rarely used and the @dh argument should generally
+ * be %NULL.
  *
  * Note that for composite types such as #GdaNumeric, #Gdate, #GdaTime, ... pointer to these
  * structures are expected, they should no be passed by value. For example:
@@ -886,7 +877,7 @@ id = gda_sql_builder_add_expr (b, NULL, G_TYPE_INT, 25);
  * Since: 4.2
  */
 GdaSqlBuilderId
-gda_sql_builder_add_expr (GdaSqlBuilder *builder, GdaDataHandler *dh, GType type, ...)
+gda_sql_builder_add_expr (GdaSqlBuilder *builder, G_GNUC_UNUSED GdaDataHandler *dh, GType type, ...)
 {
 	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
 	g_return_val_if_fail (builder->priv->main_stmt, 0);
@@ -901,7 +892,7 @@ gda_sql_builder_add_expr (GdaSqlBuilder *builder, GdaDataHandler *dh, GType type
 
 	if (!value)
 		return 0;
-	retval = gda_sql_builder_add_expr_value (builder, dh, value);
+	retval = gda_sql_builder_add_expr_value (builder, NULL, value);
 
 	gda_value_free (value);
 
@@ -911,14 +902,14 @@ gda_sql_builder_add_expr (GdaSqlBuilder *builder, GdaDataHandler *dh, GType type
 /**
  * gda_sql_builder_add_id:
  * @builder: a #GdaSqlBuilder object
- * @string: a string
+ * @str: a string
  *
  * Defines an expression representing an identifier in @builder,
  * which may be reused to build other parts of a statement,
  * for instance as a parameter to gda_sql_builder_add_cond() or
  * gda_sql_builder_add_field_value_id().
  *
- * The new expression will contain the @string literal.
+ * The new expression will contain the @str literal.
  * For example:
  * <programlisting>
  * gda_sql_builder_add_id (b, "name")
@@ -940,16 +931,16 @@ gda_sql_builder_add_expr (GdaSqlBuilder *builder, GdaDataHandler *dh, GType type
  * Since: 4.2
  */
 GdaSqlBuilderId
-gda_sql_builder_add_id (GdaSqlBuilder *builder, const gchar *string)
+gda_sql_builder_add_id (GdaSqlBuilder *builder, const gchar *str)
 {
 	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
 	g_return_val_if_fail (builder->priv->main_stmt, 0);
 
 	GdaSqlExpr *expr;
 	expr = gda_sql_expr_new (NULL);
-	if (string) {
+	if (str) {
 		expr->value = gda_value_new (G_TYPE_STRING);
-		g_value_set_string (expr->value, string);
+		g_value_set_string (expr->value, str);
 		expr->value_is_ident = (gpointer) 0x1;
 	}
 
@@ -983,7 +974,7 @@ gda_sql_builder_add_field_id (GdaSqlBuilder *builder, const gchar *field_name, c
 	if (table_name && *table_name) {
 		tmp = g_strdup_printf ("%s.%s", table_name, field_name);
 		tmp_is_allocated = TRUE;
-	}
+        }
 	else
 		tmp = (gchar*) field_name;
 
@@ -1083,7 +1074,7 @@ gda_sql_builder_add_cond (GdaSqlBuilder *builder, GdaSqlOperatorType op, GdaSqlB
 }
 
 /**
- * gda_sql_builder_add_cond_v:
+ * gda_sql_builder_add_cond_v: (skip)
  * @builder: a #GdaSqlBuilder object
  * @op: type of condition
  * @op_ids: (array length=op_ids_size): an array of ID for the arguments (not %0)
@@ -1267,7 +1258,7 @@ typedef struct {
 } BuilderJoin;
 
 /**
- * gda_sql_builder_select_join_targets:
+ * gda_sql_builder_select_join_targets: (skip)
  * @builder: a #GdaSqlBuilder object
  * @left_target_id: the ID of the left target to use (not %0)
  * @right_target_id: the ID of the right target to use (not %0)
@@ -1725,7 +1716,7 @@ gda_sql_builder_add_function_v (GdaSqlBuilder *builder, const gchar *func_name,
 }
 
 /**
- * gda_sql_builder_add_sub_select:
+ * gda_sql_builder_add_sub_select: (skip)
  * @builder: a #GdaSqlBuilder object
  * @sqlst: a pointer to a #GdaSqlStatement, which has to be a SELECT or compound SELECT. This will be copied.
  *
@@ -1788,7 +1779,7 @@ gda_sql_builder_compound_set_type (GdaSqlBuilder *builder, GdaSqlStatementCompou
 }
 
 /**
- * gda_sql_builder_compound_add_sub_select:
+ * gda_sql_builder_compound_add_sub_select: (skip)
  * @builder: a #GdaSqlBuilder object
  * @sqlst: a pointer to a #GdaSqlStatement, which has to be a SELECT or compound SELECT. This will be copied.
  *
@@ -1818,6 +1809,42 @@ gda_sql_builder_compound_add_sub_select (GdaSqlBuilder *builder, GdaSqlStatement
 	cstmt->stmt_list = g_slist_append (cstmt->stmt_list, sub);
 }
 
+
+/**
+ * gda_sql_builder_compound_add_sub_select_from_builder:
+ * @builder: a #GdaSqlBuilder object
+ * @subselect: a #GdaSqlBuilder, which has to be a SELECT or compound SELECT. This will be copied.
+ *
+ * Add a sub select to a COMPOUND statement
+ *
+ * Since: 4.2
+ */
+void
+gda_sql_builder_compound_add_sub_select_from_builder (GdaSqlBuilder *builder, GdaSqlBuilder *subselect)
+{
+	GdaSqlStatementCompound *cstmt;
+	GdaSqlStatement *sqlst;
+	GdaSqlStatement *sub;
+
+	g_return_if_fail (GDA_IS_SQL_BUILDER (builder));
+	g_return_if_fail (builder->priv->main_stmt);
+	g_return_if_fail (GDA_IS_SQL_BUILDER (subselect));
+	g_return_if_fail (subselect->priv->main_stmt);
+	if (builder->priv->main_stmt->stmt_type != GDA_SQL_STATEMENT_COMPOUND) {
+		g_warning (_("Wrong statement type"));
+		return;
+	}
+	sqlst = gda_sql_builder_get_sql_statement(subselect);
+	g_return_if_fail (sqlst);
+	g_return_if_fail ((sqlst->stmt_type == GDA_SQL_STATEMENT_SELECT) ||
+			  (sqlst->stmt_type == GDA_SQL_STATEMENT_COMPOUND));
+
+	cstmt = (GdaSqlStatementCompound*) builder->priv->main_stmt->contents;
+	sub = gda_sql_statement_copy (sqlst);
+
+	cstmt->stmt_list = g_slist_append (cstmt->stmt_list, sub);
+}
+
 /**
  * gda_sql_builder_add_case: (skip)
  * @builder: a #GdaSqlBuilder object
@@ -1946,7 +1973,7 @@ gda_sql_builder_add_case_v (GdaSqlBuilder *builder,
 }
 
 /**
- * gda_sql_builder_export_expression:
+ * gda_sql_builder_export_expression: (skip)
  * @builder: a #GdaSqlBuilder object
  * @id: the ID of the expression to be exported, (must be a valid ID in @builder, not %0)
  *
@@ -1973,7 +2000,7 @@ gda_sql_builder_export_expression (GdaSqlBuilder *builder, GdaSqlBuilderId id)
 }
 
 /**
- * gda_sql_builder_import_expression:
+ * gda_sql_builder_import_expression: (skip)
  * @builder: a #GdaSqlBuilder object
  * @expr: a #GdaSqlExpr obtained using gda_sql_builder_export_expression()
  *
@@ -1993,3 +2020,33 @@ gda_sql_builder_import_expression (GdaSqlBuilder *builder, GdaSqlExpr *expr)
 	g_return_val_if_fail (GDA_SQL_ANY_PART (expr)->type == GDA_SQL_ANY_EXPR, 0);
 	return add_part (builder, (GdaSqlAnyPart *) gda_sql_expr_copy (expr));
 }
+
+
+/**
+ * gda_sql_builder_import_expression_from_builder:
+ * @builder: a #GdaSqlBuilder object
+ * @query: a #GdaSqlBuilder object to get expression from
+ * @expr_id: a #GdaSqlBuilderId of the expression in @query
+ *
+ * Imports the an expression located in @query into @builder.
+ *
+ * Returns: the ID of the new expression, or %0 if there was an error
+ *
+ * Since: 4.2
+ */
+GdaSqlBuilderId
+gda_sql_builder_import_expression_from_builder (GdaSqlBuilder *builder, GdaSqlBuilder *query, GdaSqlBuilderId expr_id)
+{
+	GdaSqlExpr *expr;
+	
+	g_return_val_if_fail (GDA_IS_SQL_BUILDER (builder), 0);
+	g_return_val_if_fail (builder->priv->main_stmt, 0);
+	g_return_val_if_fail (GDA_IS_SQL_BUILDER (query), 0);
+	g_return_val_if_fail (query->priv->main_stmt, 0);
+	g_return_val_if_fail (expr_id, 0);
+	
+	expr = gda_sql_builder_export_expression(query, expr_id);
+	g_return_val_if_fail (GDA_SQL_ANY_PART (expr)->type == GDA_SQL_ANY_EXPR, 0);
+	return add_part (builder, (GdaSqlAnyPart *) gda_sql_expr_copy (expr));
+}
+
