@@ -66,6 +66,7 @@ static GObjectClass  *parent_class = NULL;
 
 struct _GdaStatementPrivate {
 	GdaSqlStatement *internal_struct;
+	GType           *requested_types;
 };
 
 /* signals */
@@ -178,6 +179,7 @@ gda_statement_init (GdaStatement * stmt)
 {
 	stmt->priv = g_new0 (GdaStatementPrivate, 1);
 	stmt->priv->internal_struct = NULL;
+	stmt->priv->requested_types = NULL;
 }
 
 /**
@@ -231,6 +233,10 @@ gda_statement_dispose (GObject *object)
 
 	stmt = GDA_STATEMENT (object);
 	if (stmt->priv) {
+		if (stmt->priv->requested_types) {
+			g_free (stmt->priv->requested_types);
+			stmt->priv->requested_types = NULL;
+		}
 		if (stmt->priv->internal_struct) {
 			gda_sql_statement_free (stmt->priv->internal_struct);
 			stmt->priv->internal_struct = NULL;
@@ -276,6 +282,10 @@ gda_statement_set_property (GObject *object,
 				gda_sql_statement_free (stmt->priv->internal_struct);
 				stmt->priv->internal_struct = NULL;
 			}
+			if (stmt->priv->requested_types) {
+				g_free (stmt->priv->requested_types);
+				stmt->priv->requested_types = NULL;
+			}
 			stmt->priv->internal_struct = gda_sql_statement_copy (g_value_get_pointer (value));
 			g_signal_emit (stmt, gda_statement_signals [RESET], 0);
 			break;
@@ -566,6 +576,57 @@ gda_statement_get_parameters (GdaStatement *stmt, GdaSet **out_params, GError **
 	return TRUE;
 }
 
+/*
+ * _gda_statement_get_requested_types:
+ * @stmt: a #GdaStatement
+ *
+ * Returns: a new #GType, suitable to use with gda_connection_statement_execute_select_full(), or %NULL
+ */
+const GType *
+_gda_statement_get_requested_types (GdaStatement *stmt)
+{
+	if (! stmt->priv || ! stmt->priv->internal_struct)
+		return NULL;
+	if (stmt->priv->requested_types)
+		return stmt->priv->requested_types;
+	if (stmt->priv->internal_struct->stmt_type != GDA_SQL_STATEMENT_SELECT)
+		return NULL;
+
+	GdaSqlStatementSelect *selst;
+	GSList *list;
+	GArray *array = NULL;
+ rewind:
+	selst = (GdaSqlStatementSelect*) stmt->priv->internal_struct->contents;
+	for (list = selst->expr_list; list; list = list->next) {
+		GdaSqlExpr *expr;
+		GType type = G_TYPE_INVALID;
+		expr = ((GdaSqlSelectField*) list->data)->expr;
+		if (expr->cast_as && *expr->cast_as)
+			type = gda_g_type_from_string (expr->cast_as);
+		if (array) {
+			if (type == G_TYPE_INVALID)
+				type = 0;
+			g_array_append_val (array, type);
+		}
+		else if (type != G_TYPE_INVALID) {
+			array = g_array_new (TRUE, FALSE, sizeof (GType));
+			goto rewind;
+		}
+	}
+	if (array) {
+		GType *retval;
+		guint len;
+		len = array->len;
+		retval = (GType*) g_array_free (array, FALSE);
+		retval [len] = G_TYPE_NONE;
+		stmt->priv->requested_types = retval;
+		return retval;
+	}
+	else
+		return NULL;
+}
+
+
 /*
  * SQL rendering
  */
@@ -723,9 +784,14 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
 			dh = gda_data_handler_get_default (G_VALUE_TYPE (value));
 
 		if (!dh) {
-			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-				     _("No data handler for type '%s'"), g_type_name (G_VALUE_TYPE (value)));
-			return NULL;
+			if (G_VALUE_TYPE (value) == GDA_TYPE_DEFAULT)
+				return g_strdup ("DEFAULT");
+			else {
+				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+					     _("No data handler for type '%s'"),
+					     g_type_name (G_VALUE_TYPE (value)));
+				return NULL;
+			}
 		}
 		return gda_data_handler_get_sql_from_value (dh, value);
 	}
@@ -1051,7 +1117,9 @@ default_render_compound (GdaSqlStatementCompound *stmt, GdaSqlRenderingContext *
 		case GDA_SQL_ANY_STMT_SELECT:
 			str = context->render_select (GDA_SQL_ANY_PART (sqlstmt->contents), context, error);
 			if (!str) goto err;
+			g_string_append_c (string, '(');
 			g_string_append (string, str);
+			g_string_append_c (string, ')');
 			g_free (str);
 			break;
 		case GDA_SQL_ANY_STMT_COMPOUND:
@@ -1408,44 +1476,74 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		if (!str) goto err;
 	}
 	else if (expr->value) {
-		if (expr->value_is_ident && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) &&
-		    g_value_get_string (expr->value)) {
-			gchar **ids_array;
-			gint i;
-			GString *string = NULL;
-			GdaConnectionOptions cncoptions = 0;
-			if (context->cnc)
-				g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
-
-			ids_array = gda_sql_identifier_split (g_value_get_string (expr->value));
-			if (!ids_array) 
-				str = g_value_dup_string (expr->value);
-			else if (!(ids_array[0])) goto err;
-			else {
-				for (i = 0; ids_array[i]; i++) {
-					gchar *tmp;
-					if (!string)
-						string = g_string_new ("");
-					else
-						g_string_append_c (string, '.');
-					tmp = gda_sql_identifier_quote (ids_array[i], context->cnc,
-									context->provider, FALSE,
-					   cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
-					g_string_append (string, tmp);
-					g_free (tmp);
+		if (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) {
+			/* specific treatment for strings, see documentation about GdaSqlExpr's value attribute */
+			const gchar *vstr;
+			vstr = g_value_get_string (expr->value);
+			if (vstr) {
+				if (expr->value_is_ident) {
+					gchar **ids_array;
+					gint i;
+					GString *string = NULL;
+					GdaConnectionOptions cncoptions = 0;
+					if (context->cnc)
+						g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
+					ids_array = gda_sql_identifier_split (vstr);
+					if (!ids_array)
+						str = g_strdup (vstr);
+					else if (!(ids_array[0])) goto err;
+					else {
+						for (i = 0; ids_array[i]; i++) {
+							gchar *tmp;
+							if (!string)
+								string = g_string_new ("");
+							else
+								g_string_append_c (string, '.');
+							tmp = gda_sql_identifier_quote (ids_array[i], context->cnc,
+											context->provider, FALSE,
+											cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
+							g_string_append (string, tmp);
+							g_free (tmp);
+						}
+						g_strfreev (ids_array);
+						str = g_string_free (string, FALSE);
+					}
 				}
-				g_strfreev (ids_array);
-				str = g_string_free (string, FALSE);
+				else {
+					/* we don't have an identifier */
+					if (!g_ascii_strcasecmp (vstr, "default")) {
+						if (is_default)
+							*is_default = TRUE;
+						str = g_strdup ("DEFAULT");
+					}
+					else
+						str = g_strdup (vstr);
+				}
+			}
+			else {
+				str = g_strdup ("NULL");
+				if (is_null)
+					*is_null = TRUE;
 			}
 		}
-		else {
-			str = gda_value_stringify (expr->value);
+		if (!str) {
+			/* use a GdaDataHandler to render the value as valid SQL */
+			GdaDataHandler *dh;
+			if (context->cnc) {
+				GdaServerProvider *prov;
+				prov = gda_connection_get_provider (context->cnc);
+				dh = gda_server_provider_get_data_handler_g_type (prov, context->cnc,
+										  G_VALUE_TYPE (expr->value));
+				if (!dh) goto err;
+			}
+			else
+				dh = gda_data_handler_get_default (G_VALUE_TYPE (expr->value));
+
+			if (dh)
+				str = gda_data_handler_get_sql_from_value (dh, expr->value);
+			else
+				str = gda_value_stringify (expr->value);
 			if (!str) goto err;
-			if (is_null && gda_value_is_null (expr->value))
-				*is_null = TRUE;
-			else if (is_default && (G_VALUE_TYPE (expr->value) == G_TYPE_STRING) && 
-				 !g_ascii_strcasecmp (g_value_get_string (expr->value), "default"))
-				*is_default = TRUE;
 		}
 	}
 	else if (expr->func) {
@@ -1456,8 +1554,33 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		gchar *tmp;
 		tmp = context->render_operation (GDA_SQL_ANY_PART (expr->cond), context, error);
 		if (!tmp) goto err;
-		str = g_strconcat ("(", tmp, ")", NULL);
-		g_free (tmp);
+		str = NULL;
+		if (GDA_SQL_ANY_PART (expr)->parent) {
+			if (GDA_SQL_ANY_PART (expr)->parent->type == GDA_SQL_ANY_STMT_SELECT) {
+				GdaSqlStatementSelect *selst;
+				selst = (GdaSqlStatementSelect*) (GDA_SQL_ANY_PART (expr)->parent);
+				if ((expr == selst->where_cond) ||
+				    (expr == selst->having_cond))
+					str = tmp;
+			}
+			else if (GDA_SQL_ANY_PART (expr)->parent->type == GDA_SQL_ANY_STMT_DELETE) {
+				GdaSqlStatementDelete *delst;
+				delst = (GdaSqlStatementDelete*) (GDA_SQL_ANY_PART (expr)->parent);
+				if (expr == delst->cond)
+					str = tmp;
+			}
+			else if (GDA_SQL_ANY_PART (expr)->parent->type == GDA_SQL_ANY_STMT_UPDATE) {
+				GdaSqlStatementUpdate *updst;
+				updst = (GdaSqlStatementUpdate*) (GDA_SQL_ANY_PART (expr)->parent);
+				if (expr == updst->cond)
+					str = tmp;
+			}
+		}
+
+		if (!str) {
+			str = g_strconcat ("(", tmp, ")", NULL);
+			g_free (tmp);
+		}
 	}
 	else if (expr->select) {
 		gchar *str1;
@@ -1487,10 +1610,7 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		str = g_strdup ("NULL");
 	}
 
-	if (!str) {
-		/* TO REMOVE */
-		str = g_strdup ("[...]");
-	}
+	if (!str) goto err;
 
 	if (expr->cast_as) 
 		g_string_append_printf (string, "CAST (%s AS %s)", str, expr->cast_as);
@@ -1632,7 +1752,7 @@ default_render_operation (GdaSqlOperation *op, GdaSqlRenderingContext *context,
 			goto out;
 		}
 		sqlop->sql = str;
-		if (expr->cond || expr->case_s || expr->select)
+		if (expr->case_s || expr->select)
 			sqlop->is_composed = TRUE;
 		sql_list = g_slist_prepend (sql_list, sqlop);
 	}
@@ -1861,6 +1981,16 @@ default_render_case (GdaSqlCase *case_s, GdaSqlRenderingContext *context, GError
 	return NULL;
 }
 
+static gboolean
+alias_is_quoted (const gchar *alias)
+{
+	g_assert (alias);
+	if ((*alias == '\'') || (*alias == '"'))
+		return TRUE;
+	else
+		return FALSE;
+}
+
 static gchar *
 default_render_select_field (GdaSqlSelectField *field, GdaSqlRenderingContext *context, GError **error)
 {
@@ -1879,8 +2009,21 @@ default_render_select_field (GdaSqlSelectField *field, GdaSqlRenderingContext *c
 	g_string_append (string, str);
 	g_free (str);
 
-	if (field->as)
-		g_string_append_printf (string, " AS %s", field->as);
+	if (field->as) {
+		if (! alias_is_quoted (field->as)) {
+			GdaConnectionOptions cncoptions = 0;
+			gchar *tmp;
+			if (context->cnc)
+				g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
+			tmp = gda_sql_identifier_quote (field->as, context->cnc,
+							context->provider, FALSE,
+							cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
+			g_string_append_printf (string, " AS %s", tmp);
+			g_free (tmp);
+		}
+		else
+			g_string_append_printf (string, " AS %s", field->as);
+	}
 
 	str = string->str;
 	g_string_free (string, FALSE);
@@ -1920,8 +2063,21 @@ default_render_select_target (GdaSqlSelectTarget *target, GdaSqlRenderingContext
 		g_free (str);
 	}
 
-	if (target->as)
-		g_string_append_printf (string, " AS %s", target->as);
+	if (target->as) {
+		if (! alias_is_quoted (target->as)) {
+			GdaConnectionOptions cncoptions = 0;
+			gchar *tmp;
+			if (context->cnc)
+				g_object_get (G_OBJECT (context->cnc), "options", &cncoptions, NULL);
+			tmp = gda_sql_identifier_quote (target->as, context->cnc,
+							context->provider, FALSE,
+							cncoptions & GDA_CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
+			g_string_append_printf (string, " AS %s", tmp);
+			g_free (tmp);
+		}
+		else
+			g_string_append_printf (string, " AS %s", target->as);
+	}
 
 	str = string->str;
 	g_string_free (string, FALSE);
