@@ -1,21 +1,25 @@
-/* gda-handler-bin.c
+/*
+ * Copyright (C) 2006 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 Armin Burgmeier <armin@openismus.com>
+ * Copyright (C) 2007 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  *
- * Copyright (C) 2005 - 2009 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include "gda-handler-bin.h"
@@ -67,7 +71,8 @@ gda_handler_bin_get_type (void)
 			NULL,
 			sizeof (GdaHandlerBin),
 			0,
-			(GInstanceInitFunc) gda_handler_bin_init
+			(GInstanceInitFunc) gda_handler_bin_init,
+			0
 		};		
 
 		static const GInterfaceInfo data_entry_info = {
@@ -147,11 +152,11 @@ gda_handler_bin_dispose (GObject   * object)
 }
 
 /**
- * gda_handler_bin_new
+ * gda_handler_bin_new:
  *
  * Creates a data handler for binary values
  *
- * Returns: the new object
+ * Returns: (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_bin_new (void)
